//
// Copyright (C) 2003-2004 Trevor Hogan
//

// =Xotic= Modified Source File

#ifndef TRACKER_H
 #define TRACKER_H

// Added by =Xotic=
#include "xtracker.h"
// --------------------- End of Addition

struct announce_t
{
	string strInfoHash;
	string strIP;
	string strEvent;
	unsigned int iPort;
	int64 iUploaded;
	int64 iDownloaded;
	int64 iLeft;
	string strPeerID;
};

struct torrent_t
{
	string strInfoHash;
	string strName;
	string strLowerName;
	string strFileName;
	string strAdded;
	unsigned int iSeeders;
	unsigned int iLeechers;
	unsigned int iCompleted;
	int64 iTransferred;
	int64 iSize;
	unsigned int iFiles;
	unsigned int iComments;
	int64 iAverageLeft;
	unsigned int iAverageLeftPercent;
	int64 iMinLeft;
	int64 iMaxiLeft;
	string strTag;
	string strUploader;
	string strInfoLink;
	string strIgnore;
	string strIgnored;
};

struct peer_t
{
	string strIP;
	int64 iUpped;
	int64 iDowned;
	int64 iLeft;
	unsigned long iConnected;
	float flShareRatio;
	// Added by =Xotic
	string strPeerID;
	string strClientType;
	// ----------------- End of Addition
};

/*

#define ALW_FILENAME		0		// string
#define ALW_NAME			1		// string
#define ALW_ADDED			2		// string
#define ALW_SIZE			3		// long
#define ALW_FILES			4		// int
#define ALW_FILECOMMENT		5		// string

*/

#define MAX_FILENAME_LEN	128		// user specified filename on upload
#define MAX_INFO_LINK_LEN	128		// user specified info link on upload

extern map<string, string> gmapMime;

class CTracker
{
public:
	CTracker( );
	virtual ~CTracker( );

	void saveDFile( );
	void saveComments( );
	void saveTags( );
	void saveUsers( );
	void saveRSS( string strChannelTag = string( ) );	

	void expireDownloaders( );
	void parseTorrents( const char *szDir );
	void parseTorrent( const char *szFile );
	bool checkTag( string &strTag );
	void addTag( string strInfoHash, string strTag, string strName, string strUploader, string strInfoLink, string strIgnore );
	void deleteTag( string strInfoHash );
	user_t checkUser( string strLogin, string strMD5 );
	void addUser( string strLogin, string strPass, int iAccess, string strMail );
	void deleteUser( string strLogin );
	void CountUniquePeers( );
	void AddUniquePeer( string strIP );
	void RemoveUniquePeer( string strIP );
	void Announce( struct announce_t ann );
    void RefreshFastCache( );

	void serverResponseGET( struct request_t *pRequest, struct response_t *pResponse );
	void serverResponsePOST( struct request_t *pRequest, struct response_t *pResponse, CAtomList *pPost );
	void serverResponseIndex( struct request_t *pRequest, struct response_t *pResponse );
	void serverResponseAnnounce( struct request_t *pRequest, struct response_t *pResponse );
	void serverResponseScrape( struct request_t *pRequest, struct response_t *pResponse );
	void serverResponseStats( struct request_t *pRequest, struct response_t *pResponse );
	void serverResponseTorrent( struct request_t *pRequest, struct response_t *pResponse );
	void serverResponseFile( struct request_t *pRequest, struct response_t *pResponse );
	void serverResponseRobots( struct response_t *pResponse );
	void serverResponseLogin( struct request_t *pRequest, struct response_t *pResponse );
	void serverResponseSignup( struct request_t *pRequest, struct response_t *pResponse );
	void serverResponseUploadGET( struct request_t *pRequest, struct response_t *pResponse );
	void serverResponseUploadPOST( struct request_t *pRequest, struct response_t *pResponse, CAtomList *pPost );
	void serverResponseInfo( struct request_t *pRequest, struct response_t *pResponse );
	void serverResponseAdmin( struct request_t *pRequest, struct response_t *pResponse );
	void serverResponseUsers( struct request_t *pRequest, struct response_t *pResponse );
	void serverResponseComments( struct request_t *pRequest, struct response_t *pResponse );

	void Update( );

	// Added by =Xotic=	 
	void QueueAnnounce( struct announce_t ann );
	void saveXML( );
	
	// XBNBT General
	void initXGenVars( );
	void initParseList( );
	void initTags( );
	void initXBNBTVarsList( );
	void setLinkDate( );
	void destroyXBNBT( );

#if defined XBNBT_MYSQL || BNBT_MYSQL
	// XBNBT MySQL Users Integration
	void initMySQLUsersVars( );
	void loadMySQLUsers( );
	void runLoadMySQLUsers( );
#endif
	
#ifdef XBNBT_GD
	// XBNBT Dynstat
	void initDynstatVars( );
	void decodeDynstat( );
	void saveDynstat( );
	void writeDynstatImage( );
	void runWriteDynstatImage( );
#endif

	// XBNBT Stats
	void initXStatsVars( );
	void decodeXStats( );
	void importXStats( );
	void exportXStats( );
	void saveXStats( );
	void resetXStatsDynstat( );
	void resetXStatsAnnounce( );
	void resetXStatsScrape( );
	void resetXStatsFile( );
	void resetXStatsPage( );
	void resetXStatsAll( );
#ifdef XBNBT_GD
	void tableXStatsDynstat( struct request_t *pRequest, struct response_t *pResponse );
#endif
	void tableXStatsAnnounce( struct request_t *pRequest, struct response_t *pResponse );
	void tableXStatsScrape( struct request_t *pRequest, struct response_t *pResponse );
	void tableXStatsFile( struct request_t *pRequest, struct response_t *pResponse );
	void tableXStatsPage( struct request_t *pRequest, struct response_t *pResponse );

	// XBNBT serve files
	void initServeFileVars( );

	// XBNBT Lanuage
	void initStaticLangVars( );
	
	// XBNBT CBTT
	void initCBTTVars( );

	// XBNBT HTML 4.01 Strict
	void JS_Noscript( struct response_t *pResponse );
	void JS_Target( string sRel, string sTarget, struct response_t *pResponse );
	void JS_Mouseover_Head( struct response_t *pResponse );
	void JS_Mouseover_Tail( struct response_t *pResponse );
	void JS_Valid_Check( struct response_t *pResponse );
	void HTML_Nav_Bar( struct request_t *pRequest, struct response_t *pResponse, bool bIndex, string sCSS );
	void HTML_Common_Begin( struct request_t *pRequest, struct response_t *pResponse, string sTitle, string sCSS, string sUrl, bool bIndex, int iCode );
	void HTML_Common_End( struct request_t *pRequest, struct response_t *pResponse, struct bnbttv sbtv, bool bIndex, string sCSS );
	// General
	void Button_Submit( string sID, string sLabel, struct response_t *pResponse );
	void Button_Back( string sID, string sLabel, struct response_t *pResponse );
	void Button_JS_Confirm( string sID, string sLabel, string sJSFunction, struct response_t *pResponse );
	void Button_JS_Link( string sID, string sLabel, string sJSFunction, struct response_t *pResponse );
	// Admin
	void JS_Admin( struct response_t *pResponse );
	void JS_ReturnToAdmin( struct response_t *pResponse );

	// XBNBT responses
	void serverResponseXML( struct response_t *pResponse );
	void serverResponseRSS( struct response_t *pResponse );
	void serverResponseCSS( struct response_t *pResponse );
	void serverResponseImagefill( struct response_t *pResponse );
	void serverResponseImagetrans( struct response_t *pResponse );

	// CBTT
	// Initial parse list
	void initCBTTParse( void ); 

	// Client Ban List
	int isClientBanList( string strPeerID , bool isUserAgent = false );
	void parseClientBanList( void );
	
	// CBTT IP Ban List
	int isIPBanList( string strPeerIP );
	void parseIPBanList( void );
	
	// CBTT responses
	void serverResponseIcon( struct response_t *pResponse );
	// --------------------- End of Addition

private:
	string m_strAllowedDir;
	string m_strUploadDir;
	string m_strExternalTorrentDir;
	string m_strArchiveDir;
	string m_strFileDir;
	string m_strDFile;
	string m_strCommentsFile;
	string m_strTagFile;
	string m_strUsersFile;
	string m_strStaticHeaderFile;
	string m_strStaticHeader;
	string m_strStaticFooterFile;
	string m_strStaticFooter;

	// addition by labarks

	string m_strRSSTrackerURL;
	int m_iDumpRSSFileMode;
	string m_strDumpRSSTitle;
	string m_strDumpRSSDescription;
	int m_iDumpRSS_TTL;
	string m_strDumpRSSLanguage;
	string m_strDumpRSSImageURL;
	int m_iDumpRSSImageWidth;
	int m_iDumpRSSImageHeight;
	string m_strDumpRSSCopyright;
	int m_iDumpRSSLimit;

	// end addition

	string m_strForceAnnounceURL;
	bool m_bForceAnnounceOnDL;
	int m_iParseAllowedInterval;
	int m_iSaveDFileInterval;
	int m_iDownloaderTimeOutInterval;
	int m_iRefreshStaticInterval;
	int m_iDumpRSSInterval;
	int m_iMySQLRefreshAllowedInterval;
	int m_iMySQLRefreshStatsInterval;	  
	int m_iRefreshFastCacheInterval;
	unsigned long m_iParseAllowedNext;
	unsigned long m_iSaveDFileNext;
	unsigned long m_iPrevTime;
	unsigned long m_iDownloaderTimeOutNext;
	unsigned long m_iRefreshStaticNext;
	unsigned long m_iDumpRSSNext;
	unsigned long m_iMySQLRefreshAllowedNext;
	unsigned long m_iMySQLRefreshStatsNext;
	unsigned long m_iRefreshFastCacheNext;
	int m_iAnnounceInterval;
	int m_iMinRequestInterval;
	int m_iResponseSize;
	int m_iMaxGive;
	bool m_bKeepDead;
	bool m_bAllowScrape;
	bool m_bCountUniquePeers;
	bool m_bDeleteInvalid;
	bool m_bParseOnUpload;
	int m_iMaxTorrents;
	bool m_bShowInfoHash;
	bool m_bShowNames;
	bool m_bShowStats;
	bool m_bAllowTorrentDownloads;
	bool m_bAllowComments;
	bool m_bShowAdded;
	bool m_bShowSize;
	bool m_bShowNumFiles;
	bool m_bShowCompleted;
	bool m_bShowTransferred;
	bool m_bShowMinLeft;
	bool m_bShowAverageLeft;
	bool m_bShowMaxiLeft;
	bool m_bShowLeftAsProgress;
	bool m_bShowUploader;
	bool m_bAllowInfoLink;
	bool m_bSearch;
	bool m_bSort;
	bool m_bShowFileComment;
	bool m_bShowFileContents;
	bool m_bShowShareRatios;
	bool m_bShowAvgDLRate;
	bool m_bShowAvgULRate;
	bool m_bDeleteOwnTorrents;
	bool m_bGen;
	bool m_bMySQLOverrideDState;
	int m_iPerPage;
	int m_iUsersPerPage;
	int m_iMaxPeersDisplay;
	int m_iGuestAccess;
	int m_iMemberAccess;
	int m_iFileExpires;
	int m_iNameLength;
	int m_iCommentLength;  

	vector< pair< string, string > > m_vecTags;		

	CAtomDicti *m_pAllowed;		// self.allowed
	CAtomDicti *m_pState;		// self.state
	CAtomDicti *m_pDFile;		// self.downloads
	CAtomDicti *m_pCompleted;	// self.completed
	CAtomDicti *m_pTimeDicti;	// self.times
	CAtomDicti *m_pCached;		// self.cached
	CAtomDicti *m_pComments;
	CAtomDicti *m_pTags;
	CAtomDicti *m_pUsers;
	CAtomDicti *m_pIPs;   	
	CAtomDicti *m_pFastCache;

	// Added by =Xotic=
	// XBNBT Dynstat
	CAtomDicti *m_pDynstat;
	string m_strDynstatFile;
	int m_iDynstatGenerate;
	int m_iDynstatSaveMode;
	int m_iDynstatShowLink;
	int m_iDynstatDumpInterval;
	int m_iDynstatBackground;
	int m_iDynstatXSize;
	int m_iDynstatYSize;
	int m_iDynstatFontRed;
	int m_iDynstatFontGreen;
	int m_iDynstatFontBlue;
	int m_iDynstatPNGCompress;
	int m_iDynstatJPGQuality;
	int m_iDynstatOutType;
	string m_strDynstatDumpFileDir;
	unsigned long m_strDynstatDumpNext;
	string m_strDynstatFontFile;
	string m_strDynstatLinkURL;
	int m_iDynstatSkin;
	string m_strDynstatSkinFile;
	string m_strImageOutExt;
	int m_iDynstatInType;
	
	// XBNBT MySQL Users Integration
	string m_strForumLink;
	int m_iMySQLUsersLoadInterval;
	unsigned long m_strMySQLUsersLoadNext;
	string m_strMySQLUsersLoadLast;
	string m_strMySQLUsersLoadLastStatus;
	string m_strMySQLUsersID;
	string m_strMySQLUsersGroup;
	string m_strMySQLUsersPassword;
	string m_strMySQLUsersEmail;
	string m_strMySQLUsersJoined;
	string m_strMySQLUsersIgnoreGroup1;
	string m_strMySQLUsersIgnoreGroup2;
	string m_strMySQLUsersIgnoreGroup3;
 	CAtomDicti *m_pMySQLUsersUsers;

	// The Trinity Edition 7.5r3 - Addition Begins
	// Declares new variable for "trinity_use_mouseovers"
	bool m_bUseMouseovers;
	
	// CBTT
	string m_strClientBanFile;
	int i_intBanMode;
	int i_intPeerSpoofRestrict;
	int m_iDontCompressTorrent;
	string strOverFlowLimit;
	int m_iRestrictOverflow;
	int m_iIPBanMode;
	string m_strIPBanFile;
	int m_iBlockNATedIP;
	int m_iLocalOnly;
	int m_iTorrentTraderCompatibility;
	int m_iBlacklistP2PPorts;
	CAtomList *m_pClientBannedList;   // banned Clients
	CAtomList *m_pIPBannedList;   // banned IPs
	
	// XBNBT Specific
	bool m_bBoot;
	int m_iUsersOnline;
	int m_iNavbar;
	int m_iShowPeerInfo;
	int m_iShowValidator;
	string m_strRSSValidImage;
	bool m_bFlagRSSAlert;
	bool m_bFlagXMLAlert;
	bool m_bFlagNotOwnLinkAlert;

	// language
	string m_strLogin1;
	string m_strValidate1;
	string m_strValidate2;
	string m_strJSReduce;
	string m_strJSLength;
	string m_strRSSLocalLink;

	// Mouseover
	vector< pair< string, string > > m_vecTagsMouse;

	// Serve Local
	int m_iServeLocal;
	// robots.txt
	struct localfile_t robots;
	// favicon.ico
	struct localfile_t favicon;
	// CSS
	struct localfile_t style;
	// XML
	struct localfile_t xmldump;
	// RSS
	struct localfile_t rssdump;
	// image bar fill
	struct localfile_t imagefill;
	// image bar trans
	struct localfile_t imagetrans;

	// XBNBT
	int m_iPort;
	string m_strXTrackerFQDN;
	string m_strXTrackerURL;
	string m_strXDescription;
	string m_strXTitle;
	string m_strXKeywords;

	// XBNBT Stats
	CAtomDicti *m_pXStats;
	int m_iXStatsInterval;	   
	unsigned long m_iXStatsNext;  	
	struct xbnbtstats_t m_tXStats;
	struct localfile_t statsdump;

	// queued announces
	CMutex m_mtxQueued;
	vector<struct announce_t> m_vecQueued;

	// XML
	int m_iDumpXMLInterval;	   
	unsigned long m_iDumpXMLNext;  	
	// --------------------- End of Addition
};

#endif
