/***
*
* BNBT Beta 8.0 - A C++ BitTorrent Tracker
* Copyright (C) 2003-2004 Trevor Hogan
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/

// =Xotic= Modified Source File

#include "bnbt.h"
#include "atom.h"
#include "bencode.h"
#include "sort.h"
#include "tracker.h"
#include "util.h"

// Added by =Xotic=
#include "xconfig_lang.h"
// --------------------- End of Addition

void CTracker :: serverResponseStats( struct request_t *pRequest, struct response_t *pResponse )
{
	struct bnbttv btv = UTIL_CurrentTime( );

	if( !m_bShowStats )
	{
		pResponse->strCode = "403 " + gmapLANG_CFG["server_response_403"];

		return;
	}

	if( pRequest->user.iAccess & ACCESS_VIEW )
	{
		HTML_Common_Begin(  pRequest, pResponse, gmapLANG_CFG["stats_page"], string( CSS_STATS ), string( ), NOT_INDEX, CODE_200 );
	
		string strHashString = pRequest->mapParams["info_hash"];
		string strHash = UTIL_StringToHash( strHashString );

		if( !strHash.empty( ) )
		{
			//
			// admin
			//

			if( pRequest->user.iAccess & ACCESS_EDIT )
			{
				if( m_pTags )
				{
					string strName = pRequest->mapParams["name"];
					string strTag = pRequest->mapParams["tag"];
					string strUploader = pRequest->mapParams["uploader"];
					string strInfoLink = pRequest->mapParams["infolink"];
					string strOK = pRequest->mapParams["ok"];

					if( !strName.empty( ) || !strTag.empty( ) || !strUploader.empty( ) || !strInfoLink.empty( ) )
					{
						if( strOK == "1" )
						{
							addTag( strHash, strTag, strName, strUploader, strInfoLink, string( "false" ) );

							// Modified by =Xotic=
							pResponse->strContent += "<div class=\"changed_stats\">\n";
							pResponse->strContent += "<table class=\"changed_stats\">\n";
							pResponse->strContent += "<tr>\n";
							pResponse->strContent += "<td><ul>\n";
			
							//Changed name to
							pResponse->strContent += "<li class=\"changed_stats\">" + UTIL_Xsprintf( gmapLANG_CFG["stats_changed_name"].c_str( ), UTIL_RemoveHTML( strName ).c_str( ) ) + "</li>\n";
							
							// Changed tag to
							pResponse->strContent += "<li class=\"changed_stats\">" + UTIL_Xsprintf( gmapLANG_CFG["stats_changed_tag"].c_str( ), UTIL_RemoveHTML( strTag ).c_str( ) ) + "</li>\n";
							
							// Changed uploader to
							pResponse->strContent += "<li class=\"changed_stats\">" + UTIL_Xsprintf( gmapLANG_CFG["stats_changed_uploader"].c_str( ), UTIL_RemoveHTML( strUploader ).c_str( ) ) + "</li>\n";
							
							// Changed info link to
							pResponse->strContent += "<li class=\"changed_stats\">" + UTIL_Xsprintf( gmapLANG_CFG["stats_changed_infolink"].c_str( ), UTIL_RemoveHTML( strInfoLink ).c_str( ) ) + "</li>\n";

							pResponse->strContent += "</ul></td></tr>\n";
							pResponse->strContent += "</table>\n";
							pResponse->strContent += "</div>\n";
							
							// Return to stats page
							pResponse->strContent += "<p class=\"changed_stats\">" + UTIL_Xsprintf( gmapLANG_CFG["stats_return"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["" + strName + ""] + "\" href=\"/stats.html?info_hash=" + strHashString + "\">").c_str( ), "</a>" ) + "</p>\n";

							// Output common HTML ending
							HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_STATS ) );
							// ---------------------- End of Modification

							return;
						}
						else
						{
							pResponse->strContent += "<p class=\"delete\">" + gmapLANG_CFG["stats_change_q"] + "</p>\n";
							// Added by =Xotic=
							// The Trinity Edition - Modification Begins
							// The following replaces the OK response with a YES | NO option
							// when CHANGING A TORRENT'S INFORMATION - name/uploader/infolink
							pResponse->strContent += "<p class=\"delete\"><a title=\"" + gmapLANG_CFG["yes"] + "\" href=\"/stats.html?info_hash=" + strHashString + "&amp;name=" + UTIL_StringToEscaped( strName ) + "&amp;uploader=" + UTIL_StringToEscaped( strUploader ) + "&amp;infolink=" + UTIL_StringToEscaped( strInfoLink ) + "&amp;tag=" + UTIL_StringToEscaped( strTag ) + "&amp;ok=1\">";
							pResponse->strContent += "" + gmapLANG_CFG["yes"] + "</a> | <a title=\"" + gmapLANG_CFG["no"] + "\" href=\"" + string( JS_BACK ) + "\">" + gmapLANG_CFG["no"] + "</a></p>\n";
							// ------------------------------------------------- End of Modification							

							// Modified by =Xotic=	
							// Output common HTML ending
							HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_STATS ) );
							// ---------------------- End of Modification

							return;
						}
					}

					// Added and modified by =Xotic=
					// The Trinity Edition - Modification Begins
					// Horizontal Ruled Line has a 100% width
					// Removed "(blank values mean no change)" from each field; This information now appears once after the form
					pResponse->strContent += "<div class=\"change_info\">\n";
					pResponse->strContent += "<p class=\"change_info\">" + gmapLANG_CFG["stats_change_info"] + "</p>\n";
					pResponse->strContent += "<table class=\"change_info\">\n";
					pResponse->strContent += "<tr><td>\n";
					pResponse->strContent += "<form method=\"get\" action=\"/stats.html\">\n";
					pResponse->strContent += "<p class=\"change_info_input\"><input name=\"info_hash\" type=hidden value=\"" + strHashString + "\"></p>\n";
					pResponse->strContent += "<p class=\"change_info_input\"><input id=\"name\" name=\"name\" alt=\"[" + gmapLANG_CFG["stats_new_name"] + "]\" type=text size=64 maxlength=" + CAtomInt( MAX_FILENAME_LEN ).toString( ) + "> <label for=\"name\">" + gmapLANG_CFG["stats_new_name"] + "</label> (&dagger;)</p>\n";
					pResponse->strContent += "<p class=\"change_info_input\"><input id=\"uploader\" name=\"uploader\" alt=\"[" + gmapLANG_CFG["stats_new_uploader"] + "]\" type=text size=64 maxlength=" + CAtomInt( m_iNameLength ).toString( ) + "> <label for=\"uploader\">" + gmapLANG_CFG["stats_new_uploader"] + "</label> (&dagger;&Dagger;)</p>\n";				
					pResponse->strContent += "<p class=\"change_info_input\"><input id=\"infolink\" name=\"infolink\" alt=\"[" + gmapLANG_CFG["stats_new_infolink"] + "]\" type=text size=64 maxlength=" + CAtomInt( MAX_INFO_LINK_LEN ).toString( ) + "> <label for=\"infolink\">" + gmapLANG_CFG["stats_new_infolink"] + "</label> (&dagger;&Dagger;)</p>\n";
					// ------------------------------------------------- End

					string strCur;

					CAtom *pTagInfo = m_pTags->getItem( strHash );

					if( pTagInfo && pTagInfo->isDicti( ) )
					{
						CAtom *pCur = ( (CAtomDicti *)pTagInfo )->getItem( "tag" );

						if( pCur )
							strCur = pCur->toString( );
					}

					if( !m_vecTags.empty( ) )
						pResponse->strContent += "<p class=\"change_info_tag\"><select id=\"tag\" name=\"tag\">\n";

					for( vector< pair< string, string > > :: iterator i = m_vecTags.begin( ); i != m_vecTags.end( ); i++ )
					{
						pResponse->strContent += "<option";

						if( (*i).first == strCur )
							pResponse->strContent += " selected";

						pResponse->strContent += ">" + (*i).first + "\n";
					}

					if( !m_vecTags.empty( ) )
						pResponse->strContent += "</select> <label for=\"tag\">" + gmapLANG_CFG["stats_new_tag"] + "</label><br></p>\n";
					
					pResponse->strContent += "<div class=\"change_info_button\">\n";
					Button_Submit( "submit_change", string( gmapLANG_CFG["stats_change_info"] ), pResponse );
					pResponse->strContent += "\n</div>\n";

					// Added by =Xotic=
					// The Trinity Edition - Addition Begins
					// The following explains how to leave values unchanged
					// and how to remove values from the database
					pResponse->strContent += "<p class=\"change_info_instruction\">&dagger; " + gmapLANG_CFG["stats_info_blank"] + "<br>\n";

					// Type REMOVE to clear this value from the database.
					pResponse->strContent += "&Dagger; " + UTIL_Xsprintf( gmapLANG_CFG["stats_info_remove"].c_str( ), "<span class=\"underlined\">", "</span>" ) + "</p>\n";
					// ------------------------------------------------- End of Addition					

					pResponse->strContent += "</form>\n";
					pResponse->strContent += "</td></tr>\n";
					pResponse->strContent += "</table>\n";
					pResponse->strContent += "</div>\n\n";

					// Added by =Xotic=
					// The Trinity Edition - Modification Begins
					// Changes the width of the Horizontal Ruled Line to 100%
					pResponse->strContent += "<hr class=\"stats_hr\">\n\n";																										
					// ------------------------------------------------- End of Modification

				}
			}

			// display torrent information list

			string strFileName;

			int64 iSize = 0;
			int iFiles = 1;

			if( m_pAllowed )
			{
				CAtom *pTorrent = m_pAllowed->getItem( strHash );

				if( pTorrent && dynamic_cast<CAtomList *>( pTorrent ) )
				{
					vector<CAtom *> vecTorrent = dynamic_cast<CAtomList *>( pTorrent )->getValue( );

					if( vecTorrent.size( ) == 6 )
					{
						CAtom *pFileName = vecTorrent[0];
						CAtom *pName = vecTorrent[1];
						CAtom *pAdded = vecTorrent[2];
						CAtom *pSize = vecTorrent[3];
						CAtom *pFiles = vecTorrent[4];
						CAtom *pComment = vecTorrent[5];
								
						// Modified by =Xotic=
						int iAdded = 0;
			
						pResponse->strContent += "<div class=\"stats_table\">\n";
						pResponse->strContent += "<p class=\"file_info\">" + gmapLANG_CFG["stats_file_info"] + "</p>\n";
						pResponse->strContent += "<table summary=\"file info\">\n";
					
						if( pFileName )
							strFileName = pFileName->toString( );

						if( pName )
						{
							iAdded++;
							
							if( iAdded % 2 )
								pResponse->strContent += "<tr class=\"even\">\n";
							else
								pResponse->strContent += "<tr class=\"odd\">\n";

							pResponse->strContent += "<th class=\"file_info\">" + gmapLANG_CFG["name"] + ":</th>\n";
							pResponse->strContent += "<td class=\"file_info\">" + UTIL_RemoveHTML( pName->toString( ) ) + "</td>\n";
							pResponse->strContent += "</tr>\n";
						}

						iAdded++;
							
						if( iAdded % 2 )
							pResponse->strContent += "<tr class=\"even\">\n";
						else
							pResponse->strContent += "<tr class=\"odd\">\n";

						pResponse->strContent += "<th class=\"file_info\">" + gmapLANG_CFG["info_hash"] + ":</th>\n";
						pResponse->strContent += "<td class=\"file_info\">" + strHashString + "</td>\n";
						pResponse->strContent += "</tr>\n";

							if( pAdded )
						{
							iAdded++;
							
							if( iAdded % 2 )
								pResponse->strContent += "<tr class=\"even\">\n";
							else
								pResponse->strContent += "<tr class=\"odd\">\n";

							pResponse->strContent += "<th class=\"file_info\">" + gmapLANG_CFG["added"] + ":</th>\n";
							pResponse->strContent += "<td class=\"file_info\">" + pAdded->toString( ) + "</td>\n";
							pResponse->strContent += "</tr>\n";
						}

						if( pSize && dynamic_cast<CAtomLong *>( pSize ) )
						{
							iAdded++;
							
							if( iAdded % 2 )
								pResponse->strContent += "<tr class=\"even\">\n";
							else
								pResponse->strContent += "<tr class=\"odd\">\n";

							// cache iSize

							iSize = dynamic_cast<CAtomLong *>( pSize )->getValue( );

							pResponse->strContent += "<th class=\"file_info\">" + gmapLANG_CFG["size"] + ":</th>\n";
							pResponse->strContent += "<td class=\"file_info\">" + UTIL_BytesToString( iSize ) + "</td>\n";
							pResponse->strContent += "</tr>\n";
						}

						if( pFiles && dynamic_cast<CAtomInt *>( pFiles ) )
						{
							iAdded++;
							
							if( iAdded % 2 )
								pResponse->strContent += "<tr class=\"even\">\n";
							else
								pResponse->strContent += "<tr class=\"odd\">\n";
							// cache iFiles	 

							iFiles = dynamic_cast<CAtomInt *>( pFiles )->getValue( );

							pResponse->strContent += "<th class=\"file_info\">" + gmapLANG_CFG["files"] + ":</th>\n";
							pResponse->strContent += "<td class=\"file_info\">" + pFiles->toString( ) + "</td>\n";
							pResponse->strContent += "</tr>\n";
						}

						pResponse->strContent += "</table>\n";
						pResponse->strContent += "</div>\n\n";
						// ---------------- End of Modification

						if( pComment )
						{
							if( m_bShowFileComment )
							{
								// Modified by =Xotic=
								pResponse->strContent += "<div class=\"stats_table\">\n";
								pResponse->strContent += "<p class=\"file_comment\">" + gmapLANG_CFG["file_comment"] + "</p>\n";
								pResponse->strContent += "<table summary=\"file comment\">\n";
								pResponse->strContent += "<tr class=\"file_comment\"><td class=\"file_comment\"><code>" + UTIL_RemoveHTML( pComment->toString( ) ) + "</code></td></tr>\n";
								pResponse->strContent += "</table>\n";
								pResponse->strContent += "</div>\n\n";
								// ---------------- End of Modification
							}
						}
					}
				}
			}

			pResponse->strContent += "<p class=\"stats_menu\">\n";

			if( m_pAllowed && m_bAllowTorrentDownloads && ( pRequest->user.iAccess & ACCESS_DL ) )
			{
				if( m_strExternalTorrentDir.empty( ) )
				{
					pResponse->strContent += "<a title=\"" + strHashString + ".torrent" + "\" class=\"download_link\" href=\"";
					pResponse->strContent += "/torrents/" + strHashString + ".torrent";
				}
				else
				{
					pResponse->strContent += "<a title=\"" + UTIL_StringToEscapedStrict( strFileName ) + "\" class=\"download_link\" href=\"";
					pResponse->strContent += m_strExternalTorrentDir + UTIL_StringToEscapedStrict( strFileName );
				}

				pResponse->strContent += "\">" + gmapLANG_CFG["stats_download_torrent"] + "</a>\n";
			}

			// Added by =Xotic
			// Dynstat Image Link
			// Show the link if user has Uploader or Edit privaledge
			if( m_iDynstatShowLink && m_iDynstatGenerate && !m_strDynstatFontFile.empty() && !m_strDynstatLinkURL.empty() && ( pRequest->user.iAccess & ACCESS_UPLOAD ) && ( pRequest->user.iAccess & ACCESS_EDIT ) )
			{
				switch( m_iDynstatSaveMode )
				{
				case IMAGE_BY_FILENAME:
					pResponse->strContent += "<span class=\"pipe\">|</span><a title=\"" + strFileName + m_strImageOutExt + "\" class=\"dynstat_link\" href=\"";
					pResponse->strContent += m_strDynstatLinkURL + "/" + strFileName + m_strImageOutExt;
					break;
				case IMAGE_BY_INFOHASH:
				default:
					pResponse->strContent += "<span class=\"pipe\">|</span><a title=\"" + strHashString  +  m_strImageOutExt + "\" class=\"dynstat_link\" href=\"";
					pResponse->strContent += m_strDynstatLinkURL + "/" + strHashString  +  m_strImageOutExt;
				}

				pResponse->strContent += "\">" + gmapLANG_CFG["stats_dynstat_link"] + "</a>\n";
			}
			// ---------------------------------- End of Addition

			// Added by =Xotic=
			// The Trinity Edition - Addition Begins
			// The following adds a link to DELETE TORRENT
			// when viewing the TORRENT'S STATS page
			// This link is ONLY DISPLAYED when the user had EDIT ACCESS - Moderators and Admins
			if( pRequest->user.iAccess & ACCESS_EDIT )
				pResponse->strContent += "<span class=\"pipe\">|</span><a title=\"" + gmapLANG_CFG["delete"] + ": " + strHashString + "\" class=\"delete_link\" href=\"/index.html?del=" + strHashString + "\">" + gmapLANG_CFG["stats_delete_torrent"] + "</a>\n";
			// ------------------------------------------------- End of Addition			
				
			pResponse->strContent += "</p>\n\n";
			
			if( m_bShowFileContents && iFiles > 1 )
			{
				CAtom *pDecoded = DecodeFile( ( m_strAllowedDir + strFileName ).c_str( ) );

				if( pDecoded && pDecoded->isDicti( ) )
				{
					CAtom *pInfo = ( (CAtomDicti *)pDecoded )->getItem( "info" );

					if( pInfo && pInfo->isDicti( ) )
					{
						CAtom *pFiles = ( (CAtomDicti *)pInfo )->getItem( "files" );

						if( pFiles && dynamic_cast<CAtomList *>( pFiles ) )
						{
							bool bFound = false;

							int iAdded = 0;

							vector<CAtom *> *pvecFiles = dynamic_cast<CAtomList *>( pFiles )->getValuePtr( );

							for( vector<CAtom *> :: iterator i = pvecFiles->begin( ); i != pvecFiles->end( ); i++ )
							{
								if( (*i)->isDicti( ) )
								{
									CAtom *pPath = ( (CAtomDicti *)(*i) )->getItem( "path" );
									CAtom *pLen = ( (CAtomDicti *)(*i) )->getItem( "length" );

									if( pPath && dynamic_cast<CAtomList *>( pPath ) )
									{
										if( !bFound )
										{
											// Modified by =Xotic=
											pResponse->strContent += "<div class=\"stats_table\">\n";
											pResponse->strContent += "<p class=\"contents\">" + gmapLANG_CFG["contents"] + "</p>\n";
											pResponse->strContent += "<table summary=\"contents\">\n";
											pResponse->strContent += "<tr>\n";
											pResponse->strContent += "<th class=\"path\">" + gmapLANG_CFG["file"] + "</th>\n";

											if( pLen && dynamic_cast<CAtomLong *>( pLen ) )
												pResponse->strContent += "<th class=\"bytes\">" + gmapLANG_CFG["size"] + "</th>\n";

											pResponse->strContent += "</tr>\n";
											// ---------------- End of Modification

											bFound = true;
										}

										string strPath;

										vector<CAtom *> *pvecPath = dynamic_cast<CAtomList *>( pPath )->getValuePtr( );

										for( vector<CAtom *> :: iterator j = pvecPath->begin( ); j != pvecPath->end( ); j++ )
										{
											if( j != pvecPath->begin( ) )
												strPath += '/';

											strPath += (*j)->toString( );
										}

										if( !strPath.empty( ) )
										{
											if( iAdded % 2 )
												pResponse->strContent += "<tr class=\"even\">\n";
											else
												pResponse->strContent += "<tr class=\"odd\">\n";

											pResponse->strContent += "<td class=\"path\">" + UTIL_RemoveHTML( strPath ) + "</td>\n";

											if( pLen && dynamic_cast<CAtomLong *>( pLen ) )
												pResponse->strContent += "<td class=\"bytes\">" + UTIL_BytesToString( dynamic_cast<CAtomLong *>( pLen )->getValue( ) ) + "</td>";

											pResponse->strContent += "</tr>\n";

											iAdded++;
										}
									}
								}
							}
							// Modified by =Xotic=
							if( bFound )
							{
								pResponse->strContent += "</table>\n";
								pResponse->strContent += "</div>\n\n";
							}
							// ----------------------------- End of Modification
						}
					}
				}

				if( pDecoded )
					delete pDecoded;
			}

			if( m_pDFile )
			{
				CAtom *pPeers = m_pDFile->getItem( strHash );

				if( pPeers && pPeers->isDicti( ) )
				{
					map<string, CAtom *> *pmapPeersDicti = ( (CAtomDicti *)pPeers )->getValuePtr( );

					// add the peers into this structure one by one and sort it afterwards

					struct peer_t *pPeersT = new struct peer_t[pmapPeersDicti->size( )];

					unsigned long i = 0;

					for( map<string, CAtom *> :: iterator it = pmapPeersDicti->begin( ); it != pmapPeersDicti->end( ); it++ )
					{
						pPeersT[i].iUpped = 0;
						pPeersT[i].iDowned = 0;
						pPeersT[i].iLeft = 0;
						pPeersT[i].iConnected = 0;
						pPeersT[i].flShareRatio = 0.0;

						if( (*it).second->isDicti( ) )
						{
							CAtomDicti *pPeerDicti = (CAtomDicti *)(*it).second;

							CAtom *pIP = pPeerDicti->getItem( "ip" );
							CAtom *pUpped = pPeerDicti->getItem( "uploaded" );
							CAtom *pDowned = pPeerDicti->getItem( "downloaded" );
							CAtom *pLef = pPeerDicti->getItem( "left" );
							CAtom *pConn = pPeerDicti->getItem( "connected" );

							// Added by =Xotic=
							// From jTracker 1.22
							//CAtom* pPeerID = pPeerDicti->getItem( "peer_id" );

							if( m_iShowPeerInfo )
							{
								pPeersT[i].strPeerID = UTIL_PeerID2String( (*it).first, pPeersT[i].strClientType );
								// trim peer id at 24
								if( pPeersT[i].strPeerID.size() > 24 )
									pPeersT[i].strPeerID = pPeersT[i].strPeerID.substr(0,24) + "...";
							}
							// --------------------------- End of Addition

							if( pIP )
							{
								pPeersT[i].strIP = pIP->toString( );

								// strip ip

								string :: size_type iStart = pPeersT[i].strIP.rfind( "." );

								if( iStart != string :: npos )
								{
									// don't strip ip for mods

									if( !( pRequest->user.iAccess & ACCESS_EDIT ) )
										pPeersT[i].strIP = pPeersT[i].strIP.substr( 0, iStart + 1 ) + "xxx"; 
                                }
							}

							if( pUpped && dynamic_cast<CAtomLong *>( pUpped ) )
								pPeersT[i].iUpped = dynamic_cast<CAtomLong *>( pUpped )->getValue( );

							if( pDowned && dynamic_cast<CAtomLong *>( pDowned ) )
							{
								pPeersT[i].iDowned = dynamic_cast<CAtomLong *>( pDowned )->getValue( );

								if( m_bShowShareRatios )
								{
									if( pPeersT[i].iDowned > 0 )
										pPeersT[i].flShareRatio = (float)pPeersT[i].iUpped / (float)pPeersT[i].iDowned;
									else if( pPeersT[i].iUpped == 0 )
										pPeersT[i].flShareRatio = 0.0;
									else
										pPeersT[i].flShareRatio = -1.0;
								}
							}

							if( pLef && dynamic_cast<CAtomLong *>( pLef ) )
								pPeersT[i].iLeft = dynamic_cast<CAtomLong *>( pLef )->getValue( );

							if( pConn && dynamic_cast<CAtomLong *>( pConn ) )
								pPeersT[i].iConnected = GetTime( ) - (unsigned long)dynamic_cast<CAtomLong *>( pConn )->getValue( );
						}

						i++;
					}

					string strSort = pRequest->mapParams["sort"];

					if( m_bSort )
					{
						if( !strSort.empty( ) )
						{
							int iSort = atoi( strSort.c_str( ) );

							// Modified by =Xotic=
							switch( iSort )
							{
							case SORTP_AUPPED:
								qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), asortpByUpped );
								break;
							case SORTP_ADOWNED:
								qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), asortpByDowned );
								break;
							case SORTP_ACONNECTED:
								qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), asortpByConnected );
								break;
							case SORTP_ASHARERATIO:
								if( m_bShowShareRatios )
									qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), asortpByShareRatio );
								break;
							case SORTP_ACLIENT:
								if( m_iShowPeerInfo )
									qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), asortpByClient );
								break;
							case SORTP_AIP:
								if( m_iShowPeerInfo )
									qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), asortpByIP );
								break;
							case SORTP_AAUR:
								if( m_iShowPeerInfo )
									qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), asortpByAUR );
								break;
							case SORTP_AADR:
								if( m_iShowPeerInfo )
									qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), asortpByADR );
								break;
							case SORTP_DUPPED:
								qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), dsortpByUpped );
								break;
							case SORTP_DDOWNED:
								qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), dsortpByDowned );
								break;
							case SORTP_DLEFT:
								qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), dsortpByLeft );
								break;
							case SORTP_DCONNECTED:
								qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), dsortpByConnected );
								break;
							case SORTP_DSHARERATIO:
								if( m_bShowShareRatios )
									qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), dsortpByShareRatio );
								break;
							case SORTP_DCLIENT:
								if( m_iShowPeerInfo )
									qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), dsortpByClient );
								break;
							case SORTP_DIP:
								if( m_iShowPeerInfo )
									qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), dsortpByIP );
								break;
							case SORTP_DAUR:
								if( m_iShowPeerInfo )
									qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), dsortpByAUR );
								break;
							case SORTP_DADR:
								if( m_iShowPeerInfo )
									qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), dsortpByADR );
								break;
							case SORTP_ALEFT:
							default:
								qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), asortpByLeft );
							}
							// ---------------------------- End of Modification
						}
						else
						{
							// default action is to sort by left

							qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), asortpByLeft );
					    }
					}
					else
					{
						// sort is disabled, but default action is to sort by left

						qsort( pPeersT, pmapPeersDicti->size( ), sizeof( struct peer_t ), asortpByLeft );
					}

					bool bFound = false;

					int iAdded = 0;

					//
					// seeders
					//

					string :: size_type iCountGoesHere = string :: npos;

					for( unsigned long i = 0; i < pmapPeersDicti->size( ); i++ )
					{
						if( iAdded >= m_iMaxPeersDisplay )
							break;

						if( pPeersT[i].iLeft == 0 )
						{
							if( !bFound )
							{
								// output table headers
								// Modified by =Xotic=
								pResponse->strContent += "<div class=\"stats_table\">\n";
								pResponse->strContent += "<p class=\"seeders\">" + gmapLANG_CFG["stats_seeders"] + "</p>\n";

								// to save on calculations, we're going to insert the number of seeders later, keep track of where

								iCountGoesHere = pResponse->strContent.size( ) - strlen( "</p>\n" );

								pResponse->strContent += "<table summary=\"seeders\">\n";
								pResponse->strContent += "<tr>\n";
								pResponse->strContent += "<th class=\"ip\">" + gmapLANG_CFG["peer_ip"];
								
								if( m_bSort )
								{
									pResponse->strContent += "<br>\n";
									pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_ip_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_AIP;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_ip_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_DIP;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>";
								}
								
								pResponse->strContent += "</th>\n";

								if( m_iShowPeerInfo )
								{
									pResponse->strContent += "<th class=\"client\">" + gmapLANG_CFG["client"];

									if( m_bSort )
									{
										pResponse->strContent += "<br>\n";
										pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_client_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
										pResponse->strContent += strHashString;
										pResponse->strContent += "&amp;sort=";
										pResponse->strContent += SORTPSTR_ACLIENT;
										pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_client_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
										pResponse->strContent += strHashString;
										pResponse->strContent += "&amp;sort=";
										pResponse->strContent += SORTPSTR_DCLIENT;
										pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>";
									}
								
									pResponse->strContent += "</th>\n";
								}

								pResponse->strContent += "<th class=\"bytes\">" + gmapLANG_CFG["uploaded"];
								// -------- End of Modification

								if( m_bSort )
								{
									pResponse->strContent += "<br>\n";
									pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_uploaded_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_AUPPED;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_uploaded_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_DUPPED;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>";
								}

								pResponse->strContent += "</th>\n";
								pResponse->strContent += "<th class=\"bytes\">" + gmapLANG_CFG["downloaded"];

								if( m_bSort )
								{
									pResponse->strContent += "<br>\n";
									pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_downloaded_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_ADOWNED;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_downloaded_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_DDOWNED;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>";
								}

								pResponse->strContent += "</th>\n";
								pResponse->strContent += "<th class=\"connected\">" + gmapLANG_CFG["connected"];

								if( m_bSort )
								{
									pResponse->strContent += "<br>\n";
									pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_connected_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_ACONNECTED;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_connected_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_DCONNECTED;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>";
								}

								pResponse->strContent += "</th>\n";

								if( m_bShowShareRatios )
								{
									pResponse->strContent += "<th class=\"number\">" + gmapLANG_CFG["share_ratio"];

									if( m_bSort )
									{
										pResponse->strContent += "<br>\n";
										pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_ratio_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
										pResponse->strContent += strHashString;
										pResponse->strContent += "&amp;sort=";
										pResponse->strContent += SORTPSTR_ASHARERATIO;
										pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_ratio_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
										pResponse->strContent += strHashString;
										pResponse->strContent += "&amp;sort=";
										pResponse->strContent += SORTPSTR_DSHARERATIO;
										pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>\n";
									}

									pResponse->strContent += "</th>\n";
								}

								if( m_bShowAvgULRate )
								{
									pResponse->strContent += "<th class=\"number\">" + gmapLANG_CFG["avg_up_rate"];

										if( m_bSort )
										{
											pResponse->strContent += "<br>\n";
											pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_aur_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
											pResponse->strContent += strHashString;
											pResponse->strContent += "&amp;sort=";
											pResponse->strContent += SORTPSTR_AAUR;
											pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_aur_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
											pResponse->strContent += strHashString;
											pResponse->strContent += "&amp;sort=";
											pResponse->strContent += SORTPSTR_DAUR;
											pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>";
										}

										pResponse->strContent += "</th>\n";
								}

								pResponse->strContent += "</tr>\n";

								bFound = true;
							}

							// output table rows

							if( iAdded % 2 )
								pResponse->strContent += "<tr class=\"even\">\n";
							else
								pResponse->strContent += "<tr class=\"odd\">\n";

							pResponse->strContent += "<td class=\"ip\">" + pPeersT[i].strIP + "</td>\n";

							// Added by =Xotic=
							if( m_iShowPeerInfo == 1 )
								pResponse->strContent += "<td class=\"client\">" + pPeersT[i].strClientType + "</td>\n";
							else if( m_iShowPeerInfo == 2 )
								pResponse->strContent += "<td class=\"client\">" + pPeersT[i].strPeerID + "</td>\n";
							// --------------------------------- End of Addition

							pResponse->strContent += "<td class=\"bytes\">" + UTIL_BytesToString( pPeersT[i].iUpped ) + "</td>\n";
							pResponse->strContent += "<td class=\"bytes\">" + UTIL_BytesToString( pPeersT[i].iDowned ) + "</td>\n";
							pResponse->strContent += "<td class=\"connected\">" + UTIL_SecondsToString( pPeersT[i].iConnected ) + "</td>\n";

							if( m_bShowShareRatios )
							{
								pResponse->strContent += "<td class=\"number_";

								if( pPeersT[i].flShareRatio == -1.0 )
									pResponse->strContent += "green\">";
								else if( pPeersT[i].flShareRatio < 0.8 )
									pResponse->strContent += "red\">";
								else if( pPeersT[i].flShareRatio < 1.2 )
									pResponse->strContent += "yellow\">";
								else
									pResponse->strContent += "green\">";

								// turn the share ratio into a string

								if( pPeersT[i].flShareRatio == -1.0 )
									pResponse->strContent += gmapLANG_CFG["perfect"];
								else
								{
									char szFloat[16];
									memset( szFloat, 0, sizeof( char ) * 16 );
									sprintf( szFloat, "%0.3f", pPeersT[i].flShareRatio );

									pResponse->strContent += szFloat;
								}

								pResponse->strContent += "</td>\n";
							}

							if( m_bShowAvgULRate )
							{
								pResponse->strContent += "<td class=\"number\">";

								if( pPeersT[i].iConnected > 0 )
									pResponse->strContent += UTIL_BytesToString( pPeersT[i].iUpped / pPeersT[i].iConnected ) + "/" + gmapLANG_CFG["sec"];
								else
									pResponse->strContent += gmapLANG_CFG["na"];

								pResponse->strContent += "</td>\n";
							}

							pResponse->strContent += "</tr>\n";

							iAdded++;
						}
					}

					// insert the number of seeders

					string strTemp = " (" + CAtomInt( iAdded ).toString( ) + ")";

					if( iCountGoesHere != string :: npos )
						pResponse->strContent.insert( iCountGoesHere, strTemp );

					iCountGoesHere = string :: npos;

					// Modified by =Xotic	
					if( bFound )
					{
						pResponse->strContent += "</table>\n";
						pResponse->strContent += "</div>\n\n";
					}
					// ------------------------------ End of modification

					bFound = false;

					iAdded = 0;

					//
					// leechers
					//

					for( unsigned long i = 0; i < pmapPeersDicti->size( ); i++ )
					{
						if( iAdded >= m_iMaxPeersDisplay )
							break;

						if( pPeersT[i].iLeft != 0 )
						{
							if( !bFound )
							{
								// output table headers
								// Modified by =Xotic=
								pResponse->strContent += "<div class=\"stats_table\">\n";
								pResponse->strContent += "<p class=\"leechers\">" + gmapLANG_CFG["stats_leechers"] + "</p>\n";

								// to save on calculations, we're going to insert the number of leechers later, keep track of where

								iCountGoesHere = pResponse->strContent.size( ) - strlen( "</p>\n" );

								pResponse->strContent += "<table summary=\"leechers\">\n";
								pResponse->strContent += "<tr>\n";
								pResponse->strContent += "<th class=\"ip\">" + gmapLANG_CFG["peer_ip"];
								
								if( m_bSort )
								{
									pResponse->strContent += "<br>\n";
									pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_ip_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_AIP;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_ip_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_DIP;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>";
								}
								
								pResponse->strContent += "</th>\n";

								if( m_iShowPeerInfo )
								{
									pResponse->strContent += "<th class=\"client\">" + gmapLANG_CFG["client"];
								
									if( m_bSort )
									{
										pResponse->strContent += "<br>\n";
										pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_client_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
										pResponse->strContent += strHashString;
										pResponse->strContent += "&amp;sort=";
										pResponse->strContent += SORTPSTR_ACLIENT;
										pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_client_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
										pResponse->strContent += strHashString;
										pResponse->strContent += "&amp;sort=";
										pResponse->strContent += SORTPSTR_DCLIENT;
										pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>";
									}
								
									pResponse->strContent += "</th>\n";
								}

								pResponse->strContent += "<th class=\"bytes\">" + gmapLANG_CFG["uploaded"];
								// -------------------- End of Modification

								if( m_bSort )
								{
									pResponse->strContent += "<br>\n";
									pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_uploaded_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_AUPPED;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_uploaded_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_DUPPED;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>";
								}

								pResponse->strContent += "</th>\n";
								pResponse->strContent += "<th class=\"bytes\">" + gmapLANG_CFG["downloaded"];

								if( m_bSort )
								{
									pResponse->strContent += "<br>\n";
									pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_downloaded_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_ADOWNED;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_downloaded_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_DDOWNED;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>";
								}

								if( m_pAllowed && m_bShowLeftAsProgress )
								{
									pResponse->strContent += "</th>\n";
									pResponse->strContent += "<th class=\"bytes\">" + gmapLANG_CFG["progress"];
								}
								else
								{
									pResponse->strContent += "</th>\n";
									pResponse->strContent += "<th class=\"bytes\">" + gmapLANG_CFG["left"];
								}

								if( m_bSort )
								{
									pResponse->strContent += "<br>\n";
									pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_left_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_ALEFT;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_left_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_DLEFT;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>";
								}

								pResponse->strContent += "</th>\n";
								pResponse->strContent += "<th class=\"connected\">" + gmapLANG_CFG["connected"];

								if( m_bSort )
								{
									pResponse->strContent += "<br>\n";
									pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_connected_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_ACONNECTED;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_connected_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
									pResponse->strContent += strHashString;
									pResponse->strContent += "&amp;sort=";
									pResponse->strContent += SORTPSTR_DCONNECTED;
									pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>";
								}

								pResponse->strContent += "</th>\n";

								if( m_bShowShareRatios )
								{
									pResponse->strContent += "<th class=\"number\">" + gmapLANG_CFG["share_ratio"];

									if( m_bSort )
									{
										pResponse->strContent += "<br>\n";
										pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_ratio_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
										pResponse->strContent += strHashString;
										pResponse->strContent += "&amp;sort=";
										pResponse->strContent += SORTPSTR_ASHARERATIO;
										pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_ratio_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
										pResponse->strContent += strHashString;
										pResponse->strContent += "&amp;sort=";
										pResponse->strContent += SORTPSTR_DSHARERATIO;
										pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>";
									}

									pResponse->strContent += "</th>\n";
								}

								if( m_bShowAvgULRate )
								{
									pResponse->strContent += "<th class=\"number\">" + gmapLANG_CFG["avg_up_rate"];

									if( m_bSort )
									{
										pResponse->strContent += "<br>\n";
										pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_aur_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
										pResponse->strContent += strHashString;
										pResponse->strContent += "&amp;sort=";
										pResponse->strContent += SORTPSTR_AAUR;
										pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_aur_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
										pResponse->strContent += strHashString;
										pResponse->strContent += "&amp;sort=";
										pResponse->strContent += SORTPSTR_DAUR;
										pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>";
									}

									pResponse->strContent += "</th>\n";
								}

								if( m_bShowAvgDLRate )
								{
									pResponse->strContent += "<th class=\"number\">" + gmapLANG_CFG["avg_down_rate"];

									if( m_bSort )
									{
										pResponse->strContent += "<br>\n";
										pResponse->strContent += "<a title=\"" + gmapLANG_CFG["sort_adr_ascending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
										pResponse->strContent += strHashString;
										pResponse->strContent += "&amp;sort=";
										pResponse->strContent += SORTPSTR_AADR;
										pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a title=\"" + gmapLANG_CFG["sort_adr_descending"] + "\" class=\"sort\" href=\"/stats.html?info_hash=";
										pResponse->strContent += strHashString;
										pResponse->strContent += "&amp;sort=";
										pResponse->strContent += SORTPSTR_DADR;
										pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a>";
									}

									pResponse->strContent += "</th>\n";
								}

								pResponse->strContent += "</tr>\n";

								bFound = true;
							}

							// output table rows

							if( iAdded % 2 )
								pResponse->strContent += "<tr class=\"even\">\n";
							else
								pResponse->strContent += "<tr class=\"odd\">\n";

							pResponse->strContent += "<td class=\"ip\">" + pPeersT[i].strIP + "</td>\n";

							// Added by =Xotic=
							if( m_iShowPeerInfo == 1 )
								pResponse->strContent += "<td class=\"client\">" + pPeersT[i].strClientType + "</td>\n";
							else if( m_iShowPeerInfo == 2 )
								pResponse->strContent += "<td class=\"client\">" + pPeersT[i].strPeerID + "</td>\n";
							// --------------------------------- End of Addition

							pResponse->strContent += "<td class=\"bytes\">" + UTIL_BytesToString( pPeersT[i].iUpped ) + "</td>\n";
							pResponse->strContent += "<td class=\"bytes\">" + UTIL_BytesToString( pPeersT[i].iDowned ) + "</td>\n";
							pResponse->strContent += "<td class=\"percent\">";

							if( m_pAllowed && m_bShowLeftAsProgress )
								pResponse->strContent += UTIL_BytesToString( iSize - pPeersT[i].iLeft );
							else
								pResponse->strContent += UTIL_BytesToString( pPeersT[i].iLeft );

							if( m_pAllowed )
							{
								pResponse->strContent += " (";

								int iPercent = 0;

								if( iSize > 0 )
								{
									if( m_bShowLeftAsProgress )
										iPercent = 100 - (int)( ( (float)pPeersT[i].iLeft / iSize ) * 100 );
									else
										iPercent = (int)( ( (float)pPeersT[i].iLeft / iSize ) * 100 );
								}

								pResponse->strContent += CAtomInt( iPercent ).toString( ) + "%)";

								if( !imagefill.strName.empty( ) && !imagetrans.strName.empty( ) )
								{
									pResponse->strContent += "<br>\n";

									if( iPercent > 0 )
									{
										// Modified by =Xotic=
										if( !imagefill.strURL.empty( ) )
											pResponse->strContent += "<img class=\"percent\" src=\"" + imagefill.strURL + imagefill.strName + "\" width=" + CAtomInt( iPercent ).toString( ) + " height=8 alt=\"[" + gmapLANG_CFG["completed"] + "]\" name=\"Completed\" title=\"" + gmapLANG_CFG["completed"] + "\">";
										else if( m_iServeLocal )
											pResponse->strContent += "<img class=\"percent\" src=\"" + imagefill.strName + "\" width=" + CAtomInt( iPercent ).toString( ) + " height=8 alt=\"[" + gmapLANG_CFG["completed"] + "]\" name=\"Completed\" title=\"" + gmapLANG_CFG["completed"] + "\">";
									}

									if( iPercent < 100 )
									{
										// Modified by =Xotic=
										if( !imagetrans.strURL.empty( ) )
											pResponse->strContent += "<img class=\"percent\" src=\"" + imagetrans.strURL + imagetrans.strName + "\" width=" + CAtomInt( 100 - iPercent ).toString( ) + " height=8 alt=\"[" + gmapLANG_CFG["remaining"] + "]\" name=\"Remaining\" title=\"" + gmapLANG_CFG["remaining"] + "\">";
										else if( m_iServeLocal )
											pResponse->strContent += "<img class=\"percent\" src=\"" + imagetrans.strName + "\" width=" + CAtomInt( 100 - iPercent ).toString( ) + " height=8 alt=\"[" + gmapLANG_CFG["remaining"] + "]\" name=\"Remaining\" title=\"" + gmapLANG_CFG["remaining"] + "\">";
									}								}
							}

							pResponse->strContent += "</td>\n";
							pResponse->strContent += "<td class=\"connected\">" + UTIL_SecondsToString( pPeersT[i].iConnected ) + "</td>\n";

							if( m_bShowShareRatios )
							{
								pResponse->strContent += "<td class=\"number_";

								if( pPeersT[i].flShareRatio == -1.0 )
									pResponse->strContent += "green\">";
								else if( pPeersT[i].flShareRatio < 0.8 )
									pResponse->strContent += "red\">";
								else if( pPeersT[i].flShareRatio < 1.2 )
									pResponse->strContent += "yellow\">";
								else
									pResponse->strContent += "green\">";

								// turn the share ratio into a string

								if( pPeersT[i].flShareRatio == -1.0 )
									pResponse->strContent += gmapLANG_CFG["perfect"];
								else
								{
									char szFloat[16];
									memset( szFloat, 0, sizeof( char ) * 16 );
									sprintf( szFloat, "%0.3f", pPeersT[i].flShareRatio );

									pResponse->strContent += szFloat;
								}

								pResponse->strContent += "</td>\n";
							}

							if( m_bShowAvgULRate )
							{
								pResponse->strContent += "<td class=\"number\">";

								if( pPeersT[i].iConnected > 0 )
									pResponse->strContent += UTIL_BytesToString( pPeersT[i].iUpped / pPeersT[i].iConnected ) + "/sec";
								else
									pResponse->strContent += gmapLANG_CFG["na"];

								pResponse->strContent += "</td>\n";
							}

							if( m_bShowAvgDLRate )
							{
								pResponse->strContent += "<td class=\"number\">";

								if( pPeersT[i].iConnected > 0 )
									pResponse->strContent += UTIL_BytesToString( pPeersT[i].iDowned / pPeersT[i].iConnected ) + "/sec";
								else
									pResponse->strContent += gmapLANG_CFG["na"];

								pResponse->strContent += "</td>\n";
							}

							pResponse->strContent += "</tr>\n";

							iAdded++;
						}
					}

					// insert the number of leechers

					strTemp = " (" + CAtomInt( iAdded ).toString( ) + ")";

					if( iCountGoesHere != string :: npos )
						pResponse->strContent.insert( iCountGoesHere, strTemp );

					delete [] pPeersT;

					// Modified by =Xotic=
					if( bFound )
					{
						pResponse->strContent += "</table>\n";
						pResponse->strContent += "</div>\n\n";
					}
					// ------------------------- End of modification
				}
			}
		}
	}
	else
		HTML_Common_Begin(  pRequest, pResponse, gmapLANG_CFG["stats_page"], string( CSS_STATS ), string( ), NOT_INDEX, CODE_401 );

	// Modified by =Xotic=	
	// Output common HTML ending
	HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_STATS ) );
	// ---------------------- End of Modification
}
