/***
*
* XBNBT Beta 81b.3.4 - A C++ BitTorrent Tracker
* Copyright (C) 2003-2004 =Xotic=
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/
#ifdef XBNBT_GD

#ifndef WIN32
#include <dirent.h>
#endif

#include <sys/stat.h>

#include "bnbt.h"
#include "atom.h"
#include "bencode.h"
#include "config.h"
#include "tracker.h"
#include "util.h"

// XBNBT DynStat 
#ifdef __FREEBSD__
#include </usr/local/include/gd.h> 
#else
#include <gd.h>
#endif

// XBNBT Language support
#include "xconfig_lang.h"   

// Initialise XBNBT DynStat variables
void CTracker :: initDynstatVars( )
{
	m_strDynstatFile = CFG_GetString( "xbnbt_dynstat_file", "dynstat.bnbt" );
	m_iDynstatGenerate = CFG_GetInt( "xbnbt_dynstat_generate", 0 );
	m_iDynstatSaveMode = CFG_GetInt( "xbnbt_dynstat_savemode", 0 );
	m_iDynstatShowLink = CFG_GetInt( "xbnbt_dynstat_showlink", 0 );
	m_iDynstatPNGCompress = CFG_GetInt( "xbnbt_dynstat_png_compress", -1 );
	m_iDynstatJPGQuality = CFG_GetInt( "xbnbt_dynstat_jpg_quality", -1 );
	m_iDynstatDumpInterval = CFG_GetInt( "xbnbt_dynstat_interval", 10 );
	m_iDynstatBackground = CFG_GetInt( "xbnbt_dynstat_background", 0 );
	m_iDynstatXSize = CFG_GetInt( "xbnbt_dynstat_x_size", 350 );
	m_iDynstatYSize = CFG_GetInt( "xbnbt_dynstat_y_size", 80 );
	m_iDynstatFontRed = CFG_GetInt( "xbnbt_dynstat_font_red", 255 );
	m_iDynstatFontGreen = CFG_GetInt( "xbnbt_dynstat_font_green", 255 );
	m_iDynstatFontBlue = CFG_GetInt( "xbnbt_dynstat_font_blue", 255 );
	m_iDynstatOutType = CFG_GetInt( "xbnbt_dynstat_output_type", 0 );
	m_strDynstatDumpFileDir = CFG_GetString( "xbnbt_dynstat_dir", string( ) );
	m_strDynstatFontFile = CFG_GetString( "xbnbt_dynstat_font", string( ) );
	m_strDynstatLinkURL = CFG_GetString( "xbnbt_dynstat_link", string( ) );
	m_strDynstatSkinFile = CFG_GetString( "xbnbt_dynstat_skinfile", string( ) );
	m_iDynstatSkin = CFG_GetInt( "xbnbt_dynstat_skin", 0 );
	// Set m_iDynstatInType the Dynstat image input file type
	if( !m_strDynstatSkinFile.empty( ) )
	{
		string strExt = string( );

		string :: size_type iExt = m_strDynstatSkinFile.rfind( "." );

		if( iExt != string :: npos )
			strExt = m_strDynstatSkinFile.substr( iExt );

		if( strExt == ".png" )
			m_iDynstatInType = PNG_FORMAT;
		else if( strExt == ".jpg" || strExt == ".jpeg" )
			m_iDynstatInType = JPG_FORMAT;
		else if( strExt == ".gif" )
			m_iDynstatInType = GIF_FORMAT;
		else
			m_iDynstatInType = UNK_FORMAT;
	}

	// Set m_strImageOutExt the Dynstat image filename extension
	switch( m_iDynstatOutType )
	{
	case JPG_FORMAT:
		m_strImageOutExt = ".jpg";
		break;
	case GIF_FORMAT:
		m_strImageOutExt = ".gif";
		break;
	case PNG_FORMAT:
	default:
		m_strImageOutExt = ".png";
	}
}

// XBNBT decode Dynstat frozen dictionary from file
void CTracker :: decodeDynstat( )
{
	if( m_strDynstatFile.empty( ) )
		m_pDynstat = new CAtomDicti( );
	else
	{
		CAtom *pDynstat = DecodeFile( m_strDynstatFile.c_str( ) );

		if( pDynstat && pDynstat->isDicti( ) )
			m_pDynstat = (CAtomDicti *)pDynstat;
		else
		{
			if( pDynstat )
				delete pDynstat;

			m_pDynstat = new CAtomDicti( );
		}
	}
}

// XBNBT Dynstat frozen dictionary to file
void CTracker :: saveDynstat( )
{
	string strData = Encode( m_pDynstat );

	FILE *pFile = NULL;

	if( ( pFile = fopen( m_strDynstatFile.c_str( ), "wb" ) ) == NULL )
	{
		UTIL_LogPrint( string( gmapLANG_CFG["unable_to_write_file"] + "\n" ).c_str( ), m_strDynstatFile.c_str( ) );

		return;
	}

	fwrite( (void *)strData.c_str( ), sizeof( char ), strData.size( ), pFile );
	fclose( pFile );
}

// XBNBT Dynstat
void CTracker :: writeDynstatImage( )
{
	// For elapsed time stats
	struct bnbttv btv = UTIL_CurrentTime( );

	// Check the font
	if( m_strDynstatFontFile.empty( ) )
	{
		UTIL_LogPrint( string( gmapLANG_CFG["font_file_not_set"] + "\n" ).c_str( ) );
	}
	else if( !UTIL_CheckFile ( m_strDynstatFontFile.c_str( ) ) )
	{
		UTIL_LogPrint( string( gmapLANG_CFG["font_file_not_found"] + "\n" ).c_str( ), m_strDynstatFontFile.c_str( ) );
	}
	else
	{
		// Allocate for loop
		unsigned long intLimit;
		unsigned long j = 0;    
		unsigned long i = 0;

		// Declare the images 
		gdImagePtr im = NULL; // Working image
		gdImagePtr im2 = NULL; // Master image

		// Declare output files 
		FILE *imageout = NULL;
		FILE *imagein = NULL;

		// Declare color indexes 
		int black;
		int white;
		int logobgcolor;
		int logofontcolor;
		int userfontcolor;

		// Declare size
		int iDynstatXSize;
		int iDynstatYSize;

		// Allocate the image and buffer: m_iDynstatXSize pixels across by m_iDynstatYSize pixels tall
		if ( m_iDynstatSkin )
		{
			if( m_strDynstatSkinFile.empty( ) )
			{
				// Create the Master image
				im2 = gdImageCreate( m_iDynstatXSize, m_iDynstatYSize ); // Master image
				UTIL_LogPrint( string( gmapLANG_CFG["skin_file_not_set"] + "\n" ).c_str( ) );
			}
			else if( !UTIL_CheckFile( m_strDynstatSkinFile.c_str( ) ) )
			{
				// Create the Master image
				im2 = gdImageCreate( m_iDynstatXSize, m_iDynstatYSize ); // Master image
				UTIL_LogPrint( string( gmapLANG_CFG["skin_file_not_set"] + "\n" ).c_str( ), m_strDynstatSkinFile.c_str( ) );
			}
			else
			{
				imagein = fopen( m_strDynstatSkinFile.c_str( ), "rb" );
#ifdef WIN32
				struct stat stat_buf;

				fstat( fileno( imagein ), &stat_buf );

				//Read the entire thing into a buffer that we allocate 
				char * buffer = (char *)malloc( stat_buf.st_size );
				fread( buffer, 1, stat_buf.st_size, imagein );
#endif
				switch( m_iDynstatInType )
				{
				case PNG_FORMAT:
					// Create the Master image PNG
#ifdef WIN32
					im2 = gdImageCreateFromPngPtr( stat_buf.st_size, buffer ); // Master image
#else
					im2 = gdImageCreateFromPng( imagein ); // Master image
#endif
					break;
				case JPG_FORMAT:
					// Create the Master image from JPG
#ifdef WIN32
					im2 = gdImageCreateFromJpegPtr( stat_buf.st_size, buffer ); // Master image
#else
					im2 = gdImageCreateFromJpeg( imagein ); // Master image
#endif
					break;
				case GIF_FORMAT:
					// Create the Master image from GIF
#ifdef WIN32
					im2 = gdImageCreateFromGifPtr( stat_buf.st_size, buffer ); // Master image
#else
					im2 = gdImageCreateFromGif( imagein ); // Master image
#endif
					break;
				default:
					// Create the Master image
					im2 = gdImageCreate( m_iDynstatXSize, m_iDynstatYSize ); // Master image
					UTIL_LogPrint( string( gmapLANG_CFG["skin_format_not_known"] + "\n" ).c_str( ), m_strDynstatSkinFile.c_str( ) );
				}
#ifdef WIN32
				// Free the memory
				free( buffer );
#endif
				// Close the file
				fclose( imagein );
			}
		}
		else
			// Create the Master image
			im2 = gdImageCreate( m_iDynstatXSize, m_iDynstatYSize ); // Master image


		// Set X and Y size
		iDynstatXSize = gdImageSX( im2 );
		iDynstatYSize = gdImageSY( im2 );

		// Create Working image of same size as Master image
		if( im2->trueColor )
		{
			im = gdImageCreateTrueColor( iDynstatXSize, iDynstatYSize ); // Working image

			if( gbDebug )
				UTIL_LogPrint( string( gmapLANG_CFG["skin_true_colour"] + "\n" ).c_str( ) );
		}
		else
		{
			im = gdImageCreate( iDynstatXSize, iDynstatYSize ); // Working image
	
			if( gbDebug )
				UTIL_LogPrint( string( gmapLANG_CFG["skin_pallete_colour"] + "\n" ).c_str( ) );
		}

		// Copy Master to working
		gdImageCopy(im, im2, 0, 0, 0, 0, iDynstatXSize, iDynstatYSize );

		if( im->trueColor )
		{
			// Allocate the color black (red, green and blue all minimum).
			// Since this is the first color in a new image, it will be the background color. 
			black = gdImageColorResolve(im, 0, 0, 0);  

			// Allocate the color white (red, green and blue all maximum). 
			white = gdImageColorResolve(im, 255, 255, 255);  

			// Allocate the color for the logo background 
			logobgcolor = gdImageColorResolve(im, 200, 200, 255);      

			// Allocate the color for the logo font 
			logofontcolor = gdImageColorResolve(im, 0, 0, 0);      

			// Allocate the user defined font color
			userfontcolor = gdImageColorResolve(im, m_iDynstatFontRed, m_iDynstatFontGreen, m_iDynstatFontBlue);      
		}
		else
		{
			// Allocate the color black (red, green and blue all minimum).
			// Since this is the first color in a new image, it will be the background color. 
			black = gdImageColorAllocate(im, 0, 0, 0);  

			// Allocate the color white (red, green and blue all maximum). 
			white = gdImageColorAllocate(im, 255, 255, 255);  

			// Allocate the color for the logo background 
			logobgcolor = gdImageColorAllocate(im, 200, 200, 255);      

			// Allocate the color for the logo font 
			logofontcolor = gdImageColorAllocate(im, 0, 0, 0);      

			// Allocate the user defined font color
			userfontcolor = gdImageColorResolve(im, m_iDynstatFontRed, m_iDynstatFontGreen, m_iDynstatFontBlue);      
		}

		// Start
		if( m_pDFile )
		{
			map<string, CAtom *> *pmapDicti = m_pDFile->getValuePtr( );

			unsigned long iKeySize = ( unsigned long )pmapDicti->size( );

			// add the torrents into this structure one by one and sort it afterwards

			struct torrent_t *pTorrents = new struct torrent_t[iKeySize];

			intLimit = iKeySize;

			for( map<string, CAtom *> :: iterator it = pmapDicti->begin( ); it != pmapDicti->end( ); it++ )
			{
				if( m_pAllowed )
				{
					CAtom *pList = m_pAllowed->getItem( (*it).first );

					if( pList && dynamic_cast<CAtomList *>( pList ) )
					{
						vector<CAtom *> vecTorrent = ( (CAtomList *)pList )->getValue( );

						if( vecTorrent.size( ) == 6 )
						{
							CAtom *pFileName = vecTorrent[0];
							CAtom *pName = vecTorrent[1];
							CAtom *pAdded = vecTorrent[2];
							CAtom *pSize = vecTorrent[3];
							CAtom *pFiles = vecTorrent[4];

							pTorrents[i].strInfoHash = UTIL_HashToString( (*it).first );

							if( pFileName )
								pTorrents[i].strFileName = UTIL_RemoveHTML( pFileName->toString( ) );

							if( pName )
								pTorrents[i].strName = UTIL_RemoveHTML( pName->toString( ) );

							if( pAdded )
								pTorrents[i].strAdded = pAdded->toString( );

							if( pSize )
								pTorrents[i].iSize = ( (CAtomLong *)pSize )->getValue( );

							if( pFiles )
								pTorrents[i].iFiles = ( (CAtomInt *)pFiles )->getValue( );
						}
					}
				}

				if( m_pTags )
				{
					CAtom *pDicti = m_pTags->getItem( (*it).first );

					if( pDicti && pDicti->isDicti( ) )
					{
						CAtom *pTag = ( (CAtomDicti *)pDicti )->getItem( "tag" );
						CAtom *pName = ( (CAtomDicti *)pDicti )->getItem( "name" );
						CAtom *pUploader = ( (CAtomDicti *)pDicti )->getItem( "uploader" );

						if( pTag )
							pTorrents[i].strTag = UTIL_RemoveHTML( pTag->toString( ) );

						if( pName )
							pTorrents[i].strName = UTIL_RemoveHTML( pName->toString( ) );

						if( pUploader )
							pTorrents[i].strUploader = UTIL_RemoveHTML( pUploader->toString( ) );
					}
				}

				pTorrents[i].iSeeders = 0;
				pTorrents[i].iLeechers = 0;

				CAtom *pPeers = m_pDFile->getItem( (*it).first );

				if( pPeers && pPeers->isDicti( ) )
				{
					map<string, CAtom *> *pmapPeersDicti = ( (CAtomDicti *)pPeers )->getValuePtr( );

					for( map<string, CAtom *> :: iterator k = pmapPeersDicti->begin( ); k != pmapPeersDicti->end( ); k++ )
					{
						if( (*k).second->isDicti( ) )
						{
							CAtom *pLeft = ( (CAtomDicti *)(*k).second )->getItem( "left" );

							if( pLeft && dynamic_cast<CAtomLong *>( pLeft ) )
							{
								if( ( (CAtomLong *)pLeft )->getValue( ) == 0 )
									pTorrents[i].iSeeders++;
								else
									pTorrents[i].iLeechers++;
							}
						}
					}
				}

				// Check and set frozen status
				if( m_pDynstat )
				{
					if( !m_pDynstat->getItem( (*it).first ) )
						m_pDynstat->setItem( (*it).first, new CAtomDicti( ) );

					CAtom *pDynstat = m_pDynstat->getItem( (*it).first );

					if( pDynstat && pDynstat->isDicti( ) )
					{
						CAtom *pIgnore = ( (CAtomDicti *)pDynstat )->getItem( "ignore" );
						CAtom *pIgnored = ( (CAtomDicti *)pDynstat )->getItem( "ignored" );

						if( pIgnore )
							pTorrents[i].strIgnore = pIgnore->toString( );

						if( pIgnored )
							pTorrents[i].strIgnored = pIgnored->toString( );

						if( pTorrents[i].iSeeders || pTorrents[i].iLeechers )
						{
							( (CAtomDicti *)pDynstat )->setItem( "ignore", new CAtomString( "false" ) );
							( (CAtomDicti *)pDynstat )->setItem( "ignored", new CAtomString( "false" ) );
							pTorrents[i].strIgnore = "false";
							pTorrents[i].strIgnored = "false";
						}
						else
						{
							if( pTorrents[i].strIgnore == "true" ) 
							{
								( (CAtomDicti *)pDynstat )->setItem( "ignored", new CAtomString( "true" ) );
								pTorrents[i].strIgnored = "true";
							}
							else
							{
								( (CAtomDicti *)pDynstat )->setItem( "ignored", new CAtomString( "false" ) );
								pTorrents[i].strIgnored = "false";
							}

							( (CAtomDicti *)pDynstat )->setItem( "ignore", new CAtomString( "true" ) );
							pTorrents[i].strIgnore = "true";
						}
					}
				}

				i++;
			}

			saveDynstat( );

			// Create and write the images
			struct dynstat_t imagetext;
			m_tXStats.dynstat.iFrozen = 0;
			m_tXStats.dynstat.iProcessed = 0;

			for( unsigned long i = 0; j < intLimit && i != iKeySize; i++ )
			{
				if( pTorrents[i].strIgnored == "true"  )
				{
					// Increment count and continue
					m_tXStats.dynstat.iFrozen++;
					j++;
					continue;
				}

				// Set the logo
				if( pTorrents[i].strIgnore == "true"  )
					imagetext.strLogo = UTIL_Xsprintf( gmapLANG_CFG["dynstat_logo_frozen"].c_str( ), string( XBNBT_VER ).c_str( ), UTIL_Date( ).c_str( ) );
				else
					imagetext.strLogo = UTIL_Xsprintf( gmapLANG_CFG["dynstat_logo"].c_str( ), string( XBNBT_VER ).c_str( ), UTIL_Date( ).c_str( ), CAtomInt( m_iDynstatDumpInterval ).toString( ).c_str( ) );

				// Set the title
				imagetext.strTitle = UTIL_Xsprintf( gmapLANG_CFG["dynstat_title"].c_str( ), pTorrents[i].strName.c_str( ) );

				// Set the statistics
				imagetext.strSize = UTIL_Xsprintf( gmapLANG_CFG["dynstat_size"].c_str( ), UTIL_BytesToString( pTorrents[i].iSize ).c_str( ) );
				imagetext.strFiles = UTIL_Xsprintf( gmapLANG_CFG["dynstat_files"].c_str( ), CAtomInt( pTorrents[i].iFiles ).toString( ).c_str( ) );
				imagetext.strTag = UTIL_Xsprintf( gmapLANG_CFG["dynstat_tag"].c_str( ), pTorrents[i].strTag.c_str( ) );
				imagetext.strStats = UTIL_Xsprintf( "%s    %s    %s", imagetext.strSize.c_str( ), imagetext.strFiles.c_str( ), imagetext.strTag.c_str( ) );

				//Set the info hash
				imagetext.strHash = UTIL_Xsprintf( gmapLANG_CFG["dynstat_hash"].c_str( ), pTorrents[i].strInfoHash.c_str( ) );

				// Set the filename
				imagetext.strFilename = UTIL_Xsprintf( gmapLANG_CFG["dynstat_filename"].c_str( ), pTorrents[i].strFileName.c_str( ) );

				// Set the peer details
				imagetext.strSeeders = UTIL_Xsprintf( gmapLANG_CFG["dynstat_seeders"].c_str( ), CAtomInt( pTorrents[i].iSeeders ).toString( ).c_str( ) );
				imagetext.strLeechers = UTIL_Xsprintf( gmapLANG_CFG["dynstat_leechers"].c_str( ), CAtomInt( pTorrents[i].iLeechers ).toString( ).c_str( ) );
				imagetext.strPeers = UTIL_Xsprintf( "%s    %s", imagetext.strSeeders.c_str( ), imagetext.strLeechers.c_str( ) );

				// Set image filename
				switch( m_iDynstatSaveMode )
				{
				case IMAGE_BY_FILENAME:
					imagetext.strImageFileName = pTorrents[i].strFileName + m_strImageOutExt;
					break;
				case IMAGE_BY_INFOHASH:
				default:
					imagetext.strImageFileName = pTorrents[i].strInfoHash + m_strImageOutExt;
				}

				// Set the image Path and Filename
				if( !imagetext.strImageFileName.empty( ) )
				{
					if( !m_strDynstatDumpFileDir.empty( ) )
					{
						if( UTIL_CheckDir( m_strDynstatDumpFileDir.c_str( ) ) )
						{
							imagetext.strImageFullPath = m_strDynstatDumpFileDir + "/" +  imagetext.strImageFileName;   
						}
						else
						{
							UTIL_LogPrint( string( gmapLANG_CFG["image_dir_not_found"] + "\n" ).c_str( ), m_strDynstatDumpFileDir.c_str( ) );
							continue;
						}
					}
					else
					{
						imagetext.strImageFullPath = imagetext.strImageFileName;   
						UTIL_LogPrint( string( gmapLANG_CFG["image_dir_not_set"] + "\n" ).c_str( ) );
					}
				}
				else
				{
					UTIL_LogPrint( string( gmapLANG_CFG["image_filename_empty"] + "\n" ).c_str( ) );
					continue;
				}

				// Output specified image
				// Open a Image file for writing. "wb" means "write binary", important under MSDOS, harmless under Unix. 
				imageout = fopen(imagetext.strImageFullPath.c_str( ), "wb");

				//Creating the Logo rectangle
				gdImageFilledRectangle(im, 0, iDynstatYSize - 16, iDynstatXSize - 1, iDynstatYSize - 1, logobgcolor);

				// Write the text
				int brect[8];

				gdImageStringFT( im, &brect[0], userfontcolor, (char *)m_strDynstatFontFile.c_str( ), 8.0, 0.0, 4, iDynstatYSize - 69, (char *)imagetext.strFilename.c_str( ) );
				gdImageStringFT( im, &brect[0], userfontcolor, (char *)m_strDynstatFontFile.c_str( ), 8.0, 0.0, 4, iDynstatYSize - 57, (char *)imagetext.strTitle.c_str( ) );
				gdImageStringFT( im, &brect[0], userfontcolor, (char *)m_strDynstatFontFile.c_str( ), 8.0, 0.0, 4, iDynstatYSize - 45, (char *)imagetext.strHash.c_str( ) );
				gdImageStringFT( im, &brect[0], userfontcolor, (char *)m_strDynstatFontFile.c_str( ), 8.0, 0.0, 4, iDynstatYSize - 33, (char *)imagetext.strStats.c_str( ) );
				gdImageStringFT( im, &brect[0], userfontcolor, (char *)m_strDynstatFontFile.c_str( ), 10.0, 0.0, 4, iDynstatYSize - 19, (char *)imagetext.strPeers.c_str( ) );
				gdImageStringFT( im, &brect[0], logofontcolor, (char *)m_strDynstatFontFile.c_str( ), 8.0, 0.0, 4, iDynstatYSize - 4, (char *)imagetext.strLogo.c_str( ) );
#ifdef WIN32
				int size;
				char *Data = NULL;

				// Output the image to the disk.
				switch( m_iDynstatOutType )
				{
				case JPG_FORMAT:
					// JPG format.
					Data = (char *) gdImageJpegPtr(im, &size, m_iDynstatJPGQuality);
					fwrite(Data, 1, size, imageout);
					break;
				case GIF_FORMAT:
					// GIF format.
					Data = (char *) gdImageGifPtr(im, &size);
					fwrite(Data, 1, size, imageout);
					break;
				case PNG_FORMAT:
				default:
					// PNG format.
					Data = (char *) gdImagePngPtrEx(im, &size, m_iDynstatPNGCompress);
					fwrite(Data, 1, size, imageout);
				}
#else
				// Output the image to the disk.
				switch( m_iDynstatOutType )
				{
				case JPG_FORMAT:
					// JPG format.
					gdImageJpeg(im, imageout, m_iDynstatJPGQuality);
					break;
				case GIF_FORMAT:
					// GIF format.
					gdImageGif(im, imageout);
					break;
				case PNG_FORMAT:
				default:
					// PNG format.
					gdImagePngEx(im, imageout, m_iDynstatPNGCompress);
				}
#endif

#ifdef WIN32
				// Free the memory
				gdFree(Data);  
#endif

				// Close the files. 
				fclose(imageout);

				// Reset the image
				gdImageCopy(im, im2, 0, 0, 0, 0, iDynstatXSize, iDynstatYSize );

				// Increment count
				m_tXStats.dynstat.iProcessed++;
				m_tXStats.dynstat.iTotalProcessed++;
				j++;
			}

			delete [] pTorrents;
		}

		// Destroy the images in memory. 
		gdImageDestroy(im);
		gdImageDestroy(im2);
	}
	m_tXStats.dynstat.sElapsedTime = UTIL_ElapsedTimeStr( btv, UTIL_CurrentTime( ) );
}

void CTracker :: runWriteDynstatImage( )
{
	if( m_iDynstatGenerate == 1 )
	{
		UTIL_LogPrint( string( gmapLANG_CFG["parsing_dynstat"] + "\n" ).c_str( ) );
		writeDynstatImage( );
		m_strDynstatDumpNext = GetTime( ) + m_iDynstatDumpInterval * 60;
		m_tXStats.dynstat.sLastRunTime = UTIL_Date( );
		m_tXStats.dynstat.iRun++;
	}
}
#endif
