/***
*
* XBNBT Beta 81b.3.4 - A C++ BitTorrent Tracker
* Copyright (C) 2003-2004 =Xotic=
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/

#include "bnbt.h"
#include "atom.h"
#include "bencode.h"
#include "config.h"
#include "tracker.h"
#include "util.h"

// XBNBT Language support
#include "xconfig_lang.h" 

// Initialise XBNBT Statistics variables
void CTracker :: initXStatsVars( )
{
	statsdump.strName = CFG_GetString( "xbnbt_stats_file", "stats.bnbt" );
	m_iXStatsInterval = CFG_GetInt( "xbnbt_dump_stats_interval", 600 );
	m_iXStatsNext = GetTime( ) + m_iXStatsInterval;
}

// XBNBT Stats Dictionary to file
void CTracker :: decodeXStats( )
{
	if( statsdump.strName.empty( ) )
		m_pXStats = new CAtomDicti( );
	else
	{
		CAtom *pXStats = DecodeFile( statsdump.strName.c_str( ) );

		if( pXStats && pXStats->isDicti( ) )
			m_pXStats = (CAtomDicti *)pXStats;
		else
		{
			if( pXStats )
				delete pXStats;

			m_pXStats = new CAtomDicti( );
		}
	}
}

// XBNBT Reset Dynstats Statistics
void CTracker :: resetXStatsDynstat( )
{
	m_tXStats.dynstat.iFrozen = 0;
	m_tXStats.dynstat.iProcessed = 0;
	m_tXStats.dynstat.iRun = 0;
	m_tXStats.dynstat.iTotalProcessed = 0;
	m_tXStats.dynstat.sElapsedTime = string( );
	m_tXStats.dynstat.sLastRunTime = string( );
	m_tXStats.dynstat.sLastReset = UTIL_Date( );
}

// XBNBT Reset Announce Statistics
void CTracker :: resetXStatsAnnounce( )
{
	m_tXStats.announce.iAnnounce = 0;
	m_tXStats.announce.iCompact = 0;
	m_tXStats.announce.iNopeerid = 0;
	m_tXStats.announce.iRegular = 0;
	m_tXStats.announce.iIPBanned = 0;
	m_tXStats.announce.iIPNotCleared = 0;
	//m_tXStats.announce.iIPNATBlocked = 0;
	//m_tXStats.announce.iIPLocalOnly = 0;
	m_tXStats.announce.iPeerSpoofRestrict = 0;
	m_tXStats.announce.iClientBanned = 0;
	m_tXStats.announce.iInvalidEvent = 0;
	m_tXStats.announce.iPortMissing = 0;
	m_tXStats.announce.iUploadedMissing = 0;
	m_tXStats.announce.iDownloadedMissing = 0;
	m_tXStats.announce.iLeftMissing = 0;
	m_tXStats.announce.iPeerIDLength = 0;
	m_tXStats.announce.iPortBlacklisted = 0;
	m_tXStats.announce.iDownloadedInvalid = 0;
	m_tXStats.announce.iUploadedInvalid = 0;
	m_tXStats.announce.sLastReset = UTIL_Date( );
}

// XBNBT Reset Scrape Statistics
void CTracker :: resetXStatsScrape( )
{
	m_tXStats.scrape.iScrape = 0;
	m_tXStats.scrape.iSingle = 0;
	m_tXStats.scrape.iFull = 0;
	m_tXStats.scrape.sLastReset = UTIL_Date( );
}

// XBNBT Reset File Statistics
void CTracker :: resetXStatsFile( )
{
	m_tXStats.file.iBarFill = 0;
	m_tXStats.file.iBarTrans = 0;
	m_tXStats.file.iCSS = 0;
	m_tXStats.file.iFavicon = 0;
	m_tXStats.file.iFiles = 0;
	m_tXStats.file.iRobots = 0;
	m_tXStats.file.iRSS = 0;
	m_tXStats.file.iXML = 0;
	m_tXStats.file.sLastReset = UTIL_Date( );
}

// XBNBT Reset Page Statistics
void CTracker :: resetXStatsPage( )
{
	m_tXStats.page.iAdmin = 0;
	m_tXStats.page.iComments = 0;
	m_tXStats.page.iError404 = 0;
	m_tXStats.page.iIndex = 0;
	m_tXStats.page.iInfo = 0;
	m_tXStats.page.iLogin = 0;
	m_tXStats.page.iSignup = 0;
	m_tXStats.page.iStats = 0;
	m_tXStats.page.iTorrents = 0;
	m_tXStats.page.iUpload = 0;
	m_tXStats.page.iUsers = 0;
	m_tXStats.page.sLastReset = UTIL_Date( );
}

// XBNBT Reset All Statistics
void CTracker :: resetXStatsAll( )
{
	resetXStatsDynstat( );
	resetXStatsAnnounce( );
	resetXStatsScrape( );
	resetXStatsFile( );
	resetXStatsPage( );
}

// XBNBT Stats Dictionary to file
void CTracker :: saveXStats( )
{
	string strData = Encode( m_pXStats );

	FILE *pFile = NULL;

	if( ( pFile = fopen( statsdump.strName.c_str( ), "wb" ) ) == NULL )
	{
		UTIL_LogPrint( ( gmapLANG_CFG["unable_to_write_file"] + "\n" ).c_str( ), statsdump.strName.c_str( ) );

		return;
	}

	fwrite( (void *)strData.c_str( ), sizeof( char ), strData.size( ), pFile );
	fclose( pFile );
}

// XBNBT Import Stats dictionary to struct
void CTracker :: importXStats( )
{
	if( m_pXStats )
	{
		// Announce
		if( !m_pXStats->getItem( "announce" ) )
			m_pXStats->setItem( "announce", new CAtomDicti( ) );

		CAtom *pXAnnounce = m_pXStats->getItem( "announce" );

		if( pXAnnounce && pXAnnounce->isDicti( ) )
		{
			CAtom *piAnnounce = ( (CAtomDicti *)pXAnnounce )->getItem( "iAnnounce" );
			if( piAnnounce )
				m_tXStats.announce.iAnnounce = dynamic_cast<CAtomLong *>( piAnnounce )->getValue( );
			else
				m_tXStats.announce.iAnnounce = 0;

			CAtom *piClientBanned = ( (CAtomDicti *)pXAnnounce )->getItem( "iClientBanned" );
			if( piClientBanned )
				m_tXStats.announce.iClientBanned = dynamic_cast<CAtomLong *>( piClientBanned )->getValue( );
			else
				m_tXStats.announce.iClientBanned = 0;

			CAtom *piCompact = ( (CAtomDicti *)pXAnnounce )->getItem( "iCompact" );
			if( piCompact )
				m_tXStats.announce.iCompact = dynamic_cast<CAtomLong *>( piCompact )->getValue( );
			else
				m_tXStats.announce.iCompact = 0;

			CAtom *piDownloadedInvalid = ( (CAtomDicti *)pXAnnounce )->getItem( "iDownloadedInvalid" );
			if( piDownloadedInvalid )
				m_tXStats.announce.iDownloadedInvalid = dynamic_cast<CAtomLong *>( piDownloadedInvalid )->getValue( );
			else
				m_tXStats.announce.iDownloadedInvalid = 0;

			CAtom *piDownloadedMissing = ( (CAtomDicti *)pXAnnounce )->getItem( "iDownloadedMissing" );
			if( piDownloadedMissing )
				m_tXStats.announce.iDownloadedMissing = dynamic_cast<CAtomLong *>( piDownloadedMissing )->getValue( );
			else
				m_tXStats.announce.iDownloadedMissing = 0;

			CAtom *piInvalidEvent = ( (CAtomDicti *)pXAnnounce )->getItem( "iInvalidEvent" );
			if( piInvalidEvent )
				m_tXStats.announce.iInvalidEvent = dynamic_cast<CAtomLong *>( piInvalidEvent )->getValue( );
			else
				m_tXStats.announce.iInvalidEvent = 0;

			CAtom *piIPBanned = ( (CAtomDicti *)pXAnnounce )->getItem( "iIPBanned" );
			if( piIPBanned )
				m_tXStats.announce.iIPBanned = dynamic_cast<CAtomLong *>( piIPBanned )->getValue( );
			else
				m_tXStats.announce.iIPBanned = 0;

			CAtom *piIPNotCleared = ( (CAtomDicti *)pXAnnounce )->getItem( "iIPNotCleared" );
			if( piIPNotCleared )
				m_tXStats.announce.iIPNotCleared = dynamic_cast<CAtomLong *>( piIPNotCleared )->getValue( );
			else
				m_tXStats.announce.iIPNotCleared = 0;

			CAtom *piLeftMissing = ( (CAtomDicti *)pXAnnounce )->getItem( "iLeftMissing" );
			if( piLeftMissing )
				m_tXStats.announce.iLeftMissing = dynamic_cast<CAtomLong *>( piLeftMissing )->getValue( );
			else
				m_tXStats.announce.iLeftMissing = 0;

			CAtom *piNopeerid = ( (CAtomDicti *)pXAnnounce )->getItem( "iNopeerid" );
			if( piNopeerid )
				m_tXStats.announce.iNopeerid = dynamic_cast<CAtomLong *>( piNopeerid )->getValue( );
			else
				m_tXStats.announce.iNopeerid = 0;

			CAtom *piPeerIDLength = ( (CAtomDicti *)pXAnnounce )->getItem( "iPeerIDLength" );
			if( piPeerIDLength )
				m_tXStats.announce.iPeerIDLength = dynamic_cast<CAtomLong *>( piPeerIDLength )->getValue( );
			else
				m_tXStats.announce.iPeerIDLength = 0;

			CAtom *piPeerSpoofRestrict = ( (CAtomDicti *)pXAnnounce )->getItem( "iPeerSpoofRestrict" );
			if( piPeerSpoofRestrict )
				m_tXStats.announce.iPeerSpoofRestrict = dynamic_cast<CAtomLong *>( piPeerSpoofRestrict )->getValue( );
			else
				m_tXStats.announce.iPeerSpoofRestrict = 0;

			CAtom *piPortBlacklisted = ( (CAtomDicti *)pXAnnounce )->getItem( "iPortBlacklisted" );
			if( piPortBlacklisted )
				m_tXStats.announce.iPortBlacklisted = dynamic_cast<CAtomLong *>( piPortBlacklisted )->getValue( );
			else
				m_tXStats.announce.iPortBlacklisted = 0;

			CAtom *piPortMissing = ( (CAtomDicti *)pXAnnounce )->getItem( "iPortMissing" );
			if( piPortMissing )
				m_tXStats.announce.iPortMissing = dynamic_cast<CAtomLong *>( piPortMissing )->getValue( );
			else
				m_tXStats.announce.iPortMissing = 0;

			CAtom *piRegular = ( (CAtomDicti *)pXAnnounce )->getItem( "iRegular" );
			if( piRegular )
				m_tXStats.announce.iRegular = dynamic_cast<CAtomLong *>( piRegular )->getValue( );
			else
				m_tXStats.announce.iRegular = 0;

			CAtom *piUploadedInvalid = ( (CAtomDicti *)pXAnnounce )->getItem( "iUploadedInvalid" );
			if( piUploadedInvalid )
				m_tXStats.announce.iUploadedInvalid= dynamic_cast<CAtomLong *>( piUploadedInvalid )->getValue( );
			else
				m_tXStats.announce.iUploadedInvalid= 0;

			CAtom *piUploadedMissing = ( (CAtomDicti *)pXAnnounce )->getItem( "iUploadedMissing" );
			if( piUploadedMissing )
				m_tXStats.announce.iUploadedMissing = dynamic_cast<CAtomLong *>( piUploadedMissing )->getValue( );
			else
				m_tXStats.announce.iUploadedMissing = 0;

			CAtom *psLastReset = ( (CAtomDicti *)pXAnnounce )->getItem( "sLastReset" );
			if( psLastReset )
				m_tXStats.announce.sLastReset = psLastReset->toString( );
			else
				m_tXStats.announce.sLastReset = UTIL_Date( );
		}

#ifdef XBNBT_GD
		// Dynstat
		if( !m_pXStats->getItem( "dynstat" ) )
			m_pXStats->setItem( "dynstat", new CAtomDicti( ) );

		CAtom *pXDynstat = m_pXStats->getItem( "dynstat" );
	
		if( pXDynstat && pXDynstat->isDicti( ) )
		{
			CAtom *piFrozen = ( (CAtomDicti *)pXDynstat )->getItem( "iFrozen" );
			if( piFrozen )
				m_tXStats.dynstat.iFrozen = dynamic_cast<CAtomLong *>( piFrozen )->getValue( );
			else
				m_tXStats.dynstat.iFrozen = 0;

			CAtom *piProcessed = ( (CAtomDicti *)pXDynstat )->getItem( "iProcessed" );
			if( piProcessed )
				m_tXStats.dynstat.iProcessed = dynamic_cast<CAtomLong *>( piProcessed )->getValue( );
			else
				m_tXStats.dynstat.iProcessed = 0;

			CAtom *piRun = ( (CAtomDicti *)pXDynstat )->getItem( "iRun" );
			if( piRun )
				m_tXStats.dynstat.iRun = dynamic_cast<CAtomLong *>( piRun )->getValue( );
			else
				m_tXStats.dynstat.iRun = 0;

			CAtom *piTotalProcessed = ( (CAtomDicti *)pXDynstat )->getItem( "iTotalProcessed" );
			if( piTotalProcessed )
				m_tXStats.dynstat.iTotalProcessed = dynamic_cast<CAtomLong *>( piTotalProcessed )->getValue( );
			else
				m_tXStats.dynstat.iTotalProcessed = 0;

			CAtom *psLastRunTime = ( (CAtomDicti *)pXDynstat )->getItem( "sLastRunTime" );
			if( piTotalProcessed )
				m_tXStats.dynstat.sLastRunTime = psLastRunTime->toString( );
			else
				m_tXStats.dynstat.sLastRunTime = string( );

			CAtom *psElapsedTime = ( (CAtomDicti *)pXDynstat )->getItem( "sElapsedTime" );
			if( piTotalProcessed )
				m_tXStats.dynstat.sElapsedTime = psElapsedTime->toString( );
			else
				m_tXStats.dynstat.sElapsedTime = string( );

			CAtom *psLastReset = ( (CAtomDicti *)pXDynstat )->getItem( "sLastReset" );
			if( psLastReset )
				m_tXStats.dynstat.sLastReset = psLastReset->toString( );
			else
				m_tXStats.dynstat.sLastReset = UTIL_Date( );
		}
#endif

		// File
		if( !m_pXStats->getItem( "file" ) )
			m_pXStats->setItem( "file", new CAtomDicti( ) );

		CAtom *pXFile = m_pXStats->getItem( "file" );
	
		if( pXFile && pXFile->isDicti( ) )
		{
			CAtom *piBarFill = ( (CAtomDicti *)pXFile )->getItem( "iBarFill" );
			if( piBarFill )
				m_tXStats.file.iBarFill = dynamic_cast<CAtomLong *>( piBarFill )->getValue( );
			else
				m_tXStats.file.iBarFill = 0;

			CAtom *piBarTrans = ( (CAtomDicti *)pXFile )->getItem( "iBarTrans" );
			if( piBarTrans )
				m_tXStats.file.iBarTrans = dynamic_cast<CAtomLong *>( piBarTrans )->getValue( );
			else
				m_tXStats.file.iBarTrans = 0;

			CAtom *piCSS = ( (CAtomDicti *)pXFile )->getItem( "iCSS" );
			if( piCSS )
				m_tXStats.file.iCSS = dynamic_cast<CAtomLong *>( piCSS )->getValue( );
			else
				m_tXStats.file.iCSS = 0;

			CAtom *piFavicon = ( (CAtomDicti *)pXFile )->getItem( "iFavicon" );
			if( piFavicon )
				m_tXStats.file.iFavicon = dynamic_cast<CAtomLong *>( piFavicon )->getValue( );
			else
				m_tXStats.file.iFavicon = 0;

			CAtom *piFiles = ( (CAtomDicti *)pXFile )->getItem( "iFiles" );
			if( piFiles )
				m_tXStats.file.iFiles = dynamic_cast<CAtomLong *>( piFiles )->getValue( );
			else
				m_tXStats.file.iFiles = 0;

			CAtom *piRobots = ( (CAtomDicti *)pXFile )->getItem( "iRobots" );
			if( piRobots )
				m_tXStats.file.iRobots = dynamic_cast<CAtomLong *>( piRobots )->getValue( );
			else
				m_tXStats.file.iRobots = 0;

			CAtom *piRSS = ( (CAtomDicti *)pXFile )->getItem( "iRSS" );
			if( piRSS )
				m_tXStats.file.iRSS = dynamic_cast<CAtomLong *>( piRSS )->getValue( );
			else
				m_tXStats.file.iRSS = 0;

			CAtom *piXML = ( (CAtomDicti *)pXFile )->getItem( "iXML" );
			if( piXML )
				m_tXStats.file.iXML = dynamic_cast<CAtomLong *>( piXML )->getValue( );
			else
				m_tXStats.file.iXML = 0;

			CAtom *psLastReset = ( (CAtomDicti *)pXFile )->getItem( "sLastReset" );
			if( psLastReset )
				m_tXStats.file.sLastReset = psLastReset->toString( );
			else
				m_tXStats.file.sLastReset = UTIL_Date( );
		}

		// Page
		if( !m_pXStats->getItem( "page" ) )
			m_pXStats->setItem( "page", new CAtomDicti( ) );

		CAtom *pXPage = m_pXStats->getItem( "page" );
	
		if( pXPage && pXPage->isDicti( ) )
		{
			CAtom *piAdmin = ( (CAtomDicti *)pXPage )->getItem( "iAdmin" );
			if( piAdmin )
				m_tXStats.page.iAdmin = dynamic_cast<CAtomLong *>( piAdmin )->getValue( );
			else
				m_tXStats.page.iAdmin = 0;

			CAtom *piComments = ( (CAtomDicti *)pXPage )->getItem( "iComments" );
			if( piComments )
				m_tXStats.page.iComments = dynamic_cast<CAtomLong *>( piComments )->getValue( );
			else
				m_tXStats.page.iComments = 0;

			CAtom *piError404 = ( (CAtomDicti *)pXPage )->getItem( "iError404" );
			if( piError404 )
				m_tXStats.page.iError404 = dynamic_cast<CAtomLong *>( piError404 )->getValue( );
			else
				m_tXStats.page.iError404 = 0;

			CAtom *piIndex = ( (CAtomDicti *)pXPage )->getItem( "iIndex" );
			if( piIndex )
				m_tXStats.page.iIndex = dynamic_cast<CAtomLong *>( piIndex )->getValue( );
			else
				m_tXStats.page.iIndex = 0;

			CAtom *piInfo = ( (CAtomDicti *)pXPage )->getItem( "iInfo" );
			if( piInfo )
				m_tXStats.page.iInfo = dynamic_cast<CAtomLong *>( piInfo )->getValue( );
			else
				m_tXStats.page.iInfo = 0;

			CAtom *piLogin = ( (CAtomDicti *)pXPage )->getItem( "iLogin" );
			if( piLogin )
				m_tXStats.page.iLogin = dynamic_cast<CAtomLong *>( piLogin )->getValue( );
			else
				m_tXStats.page.iLogin = 0;

			CAtom *piSignup = ( (CAtomDicti *)pXPage )->getItem( "iSignup" );
			if( piSignup )
				m_tXStats.page.iSignup = dynamic_cast<CAtomLong *>( piSignup )->getValue( );
			else
				m_tXStats.page.iSignup = 0;

			CAtom *piStats = ( (CAtomDicti *)pXPage )->getItem( "iStats" );
			if( piStats )
				m_tXStats.page.iStats = dynamic_cast<CAtomLong *>( piStats )->getValue( );
			else
				m_tXStats.page.iStats = 0;

			CAtom *piTorrents = ( (CAtomDicti *)pXPage )->getItem( "iTorrents" );
			if( piTorrents )
				m_tXStats.page.iTorrents= dynamic_cast<CAtomLong *>( piTorrents )->getValue( );
			else
				m_tXStats.page.iTorrents= 0;

			CAtom *piUpload = ( (CAtomDicti *)pXPage )->getItem( "iUpload" );
			if( piUpload )
				m_tXStats.page.iUpload = dynamic_cast<CAtomLong *>( piUpload )->getValue( );
			else
				m_tXStats.page.iUpload = 0;

			CAtom *piUsers = ( (CAtomDicti *)pXPage )->getItem( "iUsers" );
			if( piUsers )
				m_tXStats.page.iUsers = dynamic_cast<CAtomLong *>( piUsers )->getValue( );
			else
				m_tXStats.page.iUsers = 0;

			CAtom *psLastReset = ( (CAtomDicti *)pXPage )->getItem( "sLastReset" );
			if( psLastReset )
				m_tXStats.page.sLastReset = psLastReset->toString( );
			else
				m_tXStats.page.sLastReset = UTIL_Date( );
		}

		// Scrape
		if( !m_pXStats->getItem( "scrape" ) )
			m_pXStats->setItem( "scrape", new CAtomDicti( ) );

		CAtom *pXScrape = m_pXStats->getItem( "scrape" );
	
		if( pXScrape && pXScrape->isDicti( ) )
		{
			CAtom *piFull = ( (CAtomDicti *)pXScrape )->getItem( "iFull" );
			if( piFull )
				m_tXStats.scrape.iFull = dynamic_cast<CAtomLong *>( piFull )->getValue( );
			else
				m_tXStats.scrape.iFull = 0;

			CAtom *piScrape = ( (CAtomDicti *)pXScrape )->getItem( "iScrape" );
			if( piScrape )
				m_tXStats.scrape.iScrape = dynamic_cast<CAtomLong *>( piScrape )->getValue( );
			else
				m_tXStats.scrape.iScrape = 0;

			CAtom *piSingle = ( (CAtomDicti *)pXScrape )->getItem( "iSingle" );
			if( piSingle )
				m_tXStats.scrape.iSingle = dynamic_cast<CAtomLong *>( piSingle )->getValue( );
			else
				m_tXStats.scrape.iSingle = 0;

			CAtom *psLastReset = ( (CAtomDicti *)pXScrape )->getItem( "sLastReset" );
			if( psLastReset )
				m_tXStats.scrape.sLastReset = psLastReset->toString( );
			else
				m_tXStats.scrape.sLastReset = UTIL_Date( );
		}
	}
}

// XBNBT EXport Stats from struct to dictionary
void CTracker :: exportXStats( )
{
	if( m_pXStats )
	{
		if( !m_pXStats->getItem( "announce" ) )
			m_pXStats->setItem( "announce", new CAtomDicti( ) );

		CAtom *pXAnnounce = m_pXStats->getItem( "announce" );

		if( pXAnnounce && pXAnnounce->isDicti( ) )
		{
			( (CAtomDicti *)pXAnnounce )->setItem( "iAnnounce", new CAtomLong( m_tXStats.announce.iAnnounce ) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iClientBanned", new CAtomLong( m_tXStats.announce.iClientBanned ) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iCompact", new CAtomLong( m_tXStats.announce.iCompact ) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iDownloadedInvalid", new CAtomLong( m_tXStats.announce.iDownloadedInvalid) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iDownloadedMissing", new CAtomLong( m_tXStats.announce.iDownloadedMissing ) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iInvalidEvent", new CAtomLong( m_tXStats.announce.iInvalidEvent ) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iIPBanned", new CAtomLong( m_tXStats.announce.iIPBanned ) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iIPNotCleared", new CAtomLong( m_tXStats.announce.iIPNotCleared ) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iLeftMissing", new CAtomLong( m_tXStats.announce.iLeftMissing ) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iNopeerid", new CAtomLong( m_tXStats.announce.iNopeerid ) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iPeerIDLength", new CAtomLong( m_tXStats.announce.iPeerIDLength ) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iPeerSpoofRestrict", new CAtomLong( m_tXStats.announce.iPeerSpoofRestrict ) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iPortBlacklisted", new CAtomLong( m_tXStats.announce.iPortBlacklisted ) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iPortMissing", new CAtomLong( m_tXStats.announce.iPortMissing ) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iRegular", new CAtomLong( m_tXStats.announce.iRegular ) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iUploadedInvalid", new CAtomLong( m_tXStats.announce.iUploadedInvalid) );
			( (CAtomDicti *)pXAnnounce )->setItem( "iUploadedMissing", new CAtomLong( m_tXStats.announce.iUploadedMissing ) );
			( (CAtomDicti *)pXAnnounce )->setItem( "sLastReset", new CAtomString( m_tXStats.announce.sLastReset ) );
		}

#ifdef XBNBT_GD
		if( !m_pXStats->getItem( "dynstat" ) )
			m_pXStats->setItem( "dynstat", new CAtomDicti( ) );

		CAtom *pXDynstat = m_pXStats->getItem( "dynstat" );

		if( pXDynstat && pXDynstat->isDicti( ) )
		{
			( (CAtomDicti *)pXDynstat )->setItem( "iFrozen", new CAtomLong( m_tXStats.dynstat.iFrozen ) );
			( (CAtomDicti *)pXDynstat )->setItem( "iProcessed", new CAtomLong( m_tXStats.dynstat.iProcessed ) );
			( (CAtomDicti *)pXDynstat )->setItem( "iRun", new CAtomLong( m_tXStats.dynstat.iRun ) );
			( (CAtomDicti *)pXDynstat )->setItem( "iTotalProcessed", new CAtomLong( m_tXStats.dynstat.iTotalProcessed ) );
			( (CAtomDicti *)pXDynstat )->setItem( "sLastRunTime", new CAtomString( m_tXStats.dynstat.sLastRunTime ) );
			( (CAtomDicti *)pXDynstat )->setItem( "sElapsedTime", new CAtomString( m_tXStats.dynstat.sElapsedTime ) );
			( (CAtomDicti *)pXDynstat )->setItem( "sLastReset", new CAtomString( m_tXStats.dynstat.sLastReset ) );
		}
#endif

		if( !m_pXStats->getItem( "file" ) )
			m_pXStats->setItem( "file", new CAtomDicti( ) );

		CAtom *pXFile = m_pXStats->getItem( "file" );

		if( pXFile && pXFile->isDicti( ) )
		{
			( (CAtomDicti *)pXFile )->setItem( "iBarFill", new CAtomLong( m_tXStats.file.iBarFill ) );
			( (CAtomDicti *)pXFile )->setItem( "iBarTrans", new CAtomLong( m_tXStats.file.iBarTrans ) );
			( (CAtomDicti *)pXFile )->setItem( "iCSS", new CAtomLong( m_tXStats.file.iCSS ) );
			( (CAtomDicti *)pXFile )->setItem( "iFavicon", new CAtomLong( m_tXStats.file.iFavicon ) );
			( (CAtomDicti *)pXFile )->setItem( "iFiles", new CAtomLong( m_tXStats.file.iFiles ) );
			( (CAtomDicti *)pXFile )->setItem( "iRobots", new CAtomLong( m_tXStats.file.iRobots ) );
			( (CAtomDicti *)pXFile )->setItem( "iRSS", new CAtomLong( m_tXStats.file.iRSS ) );
			( (CAtomDicti *)pXFile )->setItem( "iXML", new CAtomLong( m_tXStats.file.iXML ) );
			( (CAtomDicti *)pXFile )->setItem( "sLastReset", new CAtomString( m_tXStats.file.sLastReset ) );
		}

		if( !m_pXStats->getItem( "page" ) )
			m_pXStats->setItem( "page", new CAtomDicti( ) );

		CAtom *pXPage = m_pXStats->getItem( "page" );

		if( pXPage && pXPage->isDicti( ) )
		{
			( (CAtomDicti *)pXPage )->setItem( "iAdmin", new CAtomLong( m_tXStats.page.iAdmin ) );
			( (CAtomDicti *)pXPage )->setItem( "iComments", new CAtomLong( m_tXStats.page.iComments ) );
			( (CAtomDicti *)pXPage )->setItem( "iError404", new CAtomLong( m_tXStats.page.iError404 ) );
			( (CAtomDicti *)pXPage )->setItem( "iIndex", new CAtomLong( m_tXStats.page.iIndex ) );
			( (CAtomDicti *)pXPage )->setItem( "iInfo", new CAtomLong( m_tXStats.page.iInfo ) );
			( (CAtomDicti *)pXPage )->setItem( "iLogin", new CAtomLong( m_tXStats.page.iLogin ) );
			( (CAtomDicti *)pXPage )->setItem( "iSignup", new CAtomLong( m_tXStats.page.iSignup ) );
			( (CAtomDicti *)pXPage )->setItem( "iStats", new CAtomLong( m_tXStats.page.iStats ) );
			( (CAtomDicti *)pXPage )->setItem( "iTorrents", new CAtomLong( m_tXStats.page.iTorrents) );
			( (CAtomDicti *)pXPage )->setItem( "iUpload", new CAtomLong( m_tXStats.page.iUpload ) );
			( (CAtomDicti *)pXPage )->setItem( "iUsers", new CAtomLong( m_tXStats.page.iUsers ) );
			( (CAtomDicti *)pXPage )->setItem( "sLastReset", new CAtomString( m_tXStats.page.sLastReset ) );
		}

		if( !m_pXStats->getItem( "scrape" ) )
			m_pXStats->setItem( "scrape", new CAtomDicti( ) );

		CAtom *pXScrape = m_pXStats->getItem( "scrape" );

		if( pXScrape && pXScrape->isDicti( ) )
		{
			( (CAtomDicti *)pXScrape )->setItem( "iFull", new CAtomLong( m_tXStats.scrape.iFull ) );
			( (CAtomDicti *)pXScrape )->setItem( "iScrape", new CAtomLong( m_tXStats.scrape.iScrape ) );
			( (CAtomDicti *)pXScrape )->setItem( "iSingle", new CAtomLong( m_tXStats.scrape.iSingle ) );
			( (CAtomDicti *)pXScrape )->setItem( "sLastReset", new CAtomString( m_tXStats.scrape.sLastReset ) );
		}
	}
	saveXStats( );
}

#ifdef XBNBT_GD
// XBNBT XStats table for Dynstat
void CTracker :: tableXStatsDynstat( struct request_t *pRequest, struct response_t *pResponse )
{
	pResponse->strContent += "<div class=\"admin_table\">\n";
	pResponse->strContent += "<table summary=\"xdsstats\">\n";

	if( pRequest->user.iAccess & ACCESS_ADMIN )
	{
		pResponse->strContent += "<tr><th colspan=6>" + gmapLANG_CFG["stats_dynstat"] + "/th>\n";
		pResponse->strContent += "<th colspan=1>";
		Button_JS_Confirm( "confirm_dynstat", gmapLANG_CFG["reset"], "dynstat_confirm()", pResponse );
		pResponse->strContent += "</th></tr>\n";
	}
	else
		pResponse->strContent += "<tr><th colspan=7>" + gmapLANG_CFG["stats_dynstat"] + "</th></tr>\n";

	pResponse->strContent += "<tr>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_last_reset"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_run_times"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_last_run"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_last_elapsed"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_last_processed"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_torrent_frozen"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_processed_start"] + "</th>\n";
	pResponse->strContent += "</tr>\n";
	pResponse->strContent += "<tr>\n";
	pResponse->strContent += "<td class=\"even\">" + m_tXStats.dynstat.sLastReset + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.dynstat.iRun ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + m_tXStats.dynstat.sLastRunTime + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + m_tXStats.dynstat.sElapsedTime + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.dynstat.iProcessed ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.dynstat.iFrozen ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.dynstat.iTotalProcessed ).toString( ) + "</td>\n";
	pResponse->strContent += "</tr>\n";

	pResponse->strContent += "</table>\n";
	pResponse->strContent += "</div>\n\n";
}
#endif

// XBNBT XStats table for Announce
void CTracker :: tableXStatsAnnounce( struct request_t *pRequest, struct response_t *pResponse )
{
	pResponse->strContent += "<div class=\"admin_table\">\n";
	pResponse->strContent += "<table summary=\"xannstats\">\n";

	if( pRequest->user.iAccess & ACCESS_ADMIN )
	{
		pResponse->strContent += "<tr><th colspan=8>" + gmapLANG_CFG["stats_announce"] + "</th>\n";
		pResponse->strContent += "<th colspan=1>";
		Button_JS_Confirm( "confirm_announce", gmapLANG_CFG["reset"], "announce_confirm()", pResponse );
		pResponse->strContent += "</th></tr>\n";
	}
	else
	{
		pResponse->strContent += "<tr><th colspan=9>" + gmapLANG_CFG["stats_announce"] + "</th></tr>\n";
	}

	pResponse->strContent += "<tr>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_last_reset"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_announces"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_compact"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_no_peer_id"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_regular"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_ip_banned"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_ip_not_cleared"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_peer_spoof_blocked"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_client_banned"] + "</th>\n";
	pResponse->strContent += "</tr>\n";

	pResponse->strContent += "<tr>\n";
	pResponse->strContent += "<td class=\"even\">" + m_tXStats.announce.sLastReset + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.announce.iAnnounce ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.announce.iCompact ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.announce.iNopeerid ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.announce.iRegular ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.announce.iIPBanned ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.announce.iIPNotCleared ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.announce.iPeerSpoofRestrict ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.announce.iClientBanned ).toString( ) + "</td>\n";
	pResponse->strContent += "</tr>\n";

	pResponse->strContent += "<tr>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_invalid_event"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_port_missing"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_uploaded_missing"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_downloaded_missing"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_left_missing"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_peer_id_length"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_port_blacklisted"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_download_invalid"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_upload_invalid"] + "</th>\n";
	pResponse->strContent += "</tr>\n";

	pResponse->strContent += "<tr>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.announce.iInvalidEvent ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.announce.iPortMissing ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.announce.iUploadedMissing ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.announce.iDownloadedMissing ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.announce.iLeftMissing ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.announce.iPeerIDLength ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.announce.iPortBlacklisted ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.announce.iDownloadedInvalid ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.announce.iUploadedInvalid ).toString( ) + "</td>\n";
	pResponse->strContent += "</tr>\n";

	pResponse->strContent += "</table>\n";
	pResponse->strContent += "</div>\n\n";
}

// XBNBT XStats table for Scrape
void CTracker :: tableXStatsScrape( struct request_t *pRequest, struct response_t *pResponse )
{
	pResponse->strContent += "<div class=\"admin_table\">\n";
	pResponse->strContent += "<table summary=\"xscrstats\">\n";

	if( pRequest->user.iAccess & ACCESS_ADMIN )
	{
		pResponse->strContent += "<tr><th colspan=3>" + gmapLANG_CFG["stats_scrape"] + "</th>\n";
		pResponse->strContent += "<th colspan=1>";
		Button_JS_Confirm( "confirm_scrape", gmapLANG_CFG["reset"], "scrape_confirm()", pResponse );
		pResponse->strContent += "</th></tr>\n";
	}
	else
	{
		pResponse->strContent += "<tr><th colspan=4>" + gmapLANG_CFG["stats_scrape"] + "</th></tr>\n";
	}

	pResponse->strContent += "<tr>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_last_reset"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_scrapes"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_single"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_full"] + "</th>\n";
	pResponse->strContent += "</tr>\n";
	pResponse->strContent += "<tr>\n";
	pResponse->strContent += "<td class=\"even\">" + m_tXStats.scrape.sLastReset + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.scrape.iScrape ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.scrape.iSingle ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.scrape.iFull ).toString( ) + "</td>\n";
	pResponse->strContent += "</tr>\n";

	pResponse->strContent += "</table>\n";
	pResponse->strContent += "</div>\n\n";
}

// XBNBT XStats table for File
void CTracker :: tableXStatsFile( struct request_t *pRequest, struct response_t *pResponse )
{
	pResponse->strContent += "<div class=\"admin_table\">\n";
	pResponse->strContent += "<table summary=\"xfilestats\">\n";

	if( pRequest->user.iAccess & ACCESS_ADMIN )
	{
		pResponse->strContent += "<tr><th colspan=8>" + gmapLANG_CFG["stats_file"] + "</th>\n";
		pResponse->strContent += "<th colspan=1>";
		Button_JS_Confirm( "confirm_file", gmapLANG_CFG["reset"], "file_confirm()", pResponse );
		pResponse->strContent += "</th></tr>\n";
	}
	else
	{
		pResponse->strContent += "<tr><th colspan=9>" + gmapLANG_CFG["stats_file"] + "</th></tr>\n";
	}

	pResponse->strContent += "<tr>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_last_reset"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_favicon"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_ibfill"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_ibtrans"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_css"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_rss"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_xml"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_robots"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_other"] + "</th>\n";
	pResponse->strContent += "</tr>\n";
	pResponse->strContent += "<tr>\n";
	pResponse->strContent += "<td class=\"even\">" + m_tXStats.file.sLastReset + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.file.iFavicon ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.file.iBarFill ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.file.iBarTrans ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.file.iCSS ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.file.iRSS ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.file.iXML ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.file.iRobots ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.file.iFiles ).toString( ) + "</td>\n";
	pResponse->strContent += "</tr>\n";

	pResponse->strContent += "</table>\n";
	pResponse->strContent += "</div>\n\n";
}

// XBNBT XStats table for Page
void CTracker :: tableXStatsPage( struct request_t *pRequest, struct response_t *pResponse )
{
	pResponse->strContent += "<div class=\"admin_table\">\n";
	pResponse->strContent += "<table summary=\"xpagestats\">\n";

	if( pRequest->user.iAccess & ACCESS_ADMIN )
	{
		pResponse->strContent += "<tr><th colspan=11>" + gmapLANG_CFG[""] + "XBNBT HTML Page Stats</th>\n";
		pResponse->strContent += "<th colspan=1>";
		Button_JS_Confirm( "confirm_page", gmapLANG_CFG["reset"], "page_confirm()", pResponse );
		pResponse->strContent += "</th></tr>\n";
	}
	else
	{
		pResponse->strContent += "<tr><th colspan=12>" + gmapLANG_CFG[""] + "XBNBT HTML Page Stats</th></tr>\n";
	}

	pResponse->strContent += "<tr>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_last_reset"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_index"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_stats"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_comments"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_info"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_login"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_signup"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_upload"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_torrents"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_admin"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_users"] + "</th>\n";
	pResponse->strContent += "<th>" + gmapLANG_CFG["stats_error404"] + "</th>\n";
	pResponse->strContent += "</tr>\n";

	pResponse->strContent += "<tr>\n";
	pResponse->strContent += "<td class=\"even\">" + m_tXStats.page.sLastReset + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.page.iIndex ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.page.iStats ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.page.iComments ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.page.iInfo ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.page.iLogin ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.page.iSignup ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.page.iUpload ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.page.iTorrents ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.page.iAdmin ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"even\">" + CAtomLong( m_tXStats.page.iUsers ).toString( ) + "</td>\n";
	pResponse->strContent += "<td class=\"odd\">" + CAtomLong( m_tXStats.page.iError404 ).toString( ) + "</td>\n";
	pResponse->strContent += "</tr>\n";

	pResponse->strContent += "</table>\n";
	pResponse->strContent += "</div>\n\n";
}
